@@ -28,20 +28,19 @@ import               webbrowser
 import               wx
 import jinja2     as j2
 
-import fsleyes_widgets.utils.progress     as progress
-import fsleyes_widgets.utils.status       as status
-import fsl.utils.settings                 as settings
-from   fsl.utils.platform import platform as fslplatform
-import fsl.utils.tempdir                  as tempdir
-import fsl.utils.idle                     as idle
+import fsleyes_widgets.utils.progress as progress
+import fsleyes_widgets.utils.status   as status
+import fsl.utils.settings             as settings
+import fsl.utils.tempdir              as tempdir
+import fsl.utils.idle                 as idle
 
-import                                       fsleyes
-import fsleyes.main                       as fsleyes_main
-import fsleyes.strings                    as strings
-import fsleyes.actions.screenshot         as screenshot
+import                                   fsleyes
+import fsleyes.main                   as fsleyes_main
+import fsleyes.strings                as strings
+import fsleyes.actions.screenshot     as screenshot
 
-from . import                                base
-from . import                                runscript
+import fsleyes.actions.base           as base
+import fsleyes.actions.runscript      as runscript
 
 try:
     import                            zmq
@@ -212,7 +211,7 @@ class NotebookAction(base.Action):
         return server
 
 
-class BackgroundIPythonKernel(object):
+class BackgroundIPythonKernel:
     """The BackgroundIPythonKernel creates an IPython jupyter kernel and makes
     it accessible over tcp (on the local machine only).
 
@@ -380,8 +379,12 @@ class BackgroundIPythonKernel(object):
         return self.__error
 
 
-    def __screenshot(self, view):
-        """Insert a screenshot of the given view into the notebook. """
+    def __screenshot(self, view=None):
+        """Insert a screenshot of the given view panel into the notebook.
+        If ``view`` is not specified, the first view is assumed.
+        """
+        if view is None:
+            view = self.__frame.viewPanels[0]
         with tempdir.tempdir():
             screenshot.screenshot(view, 'screenshot.png')
             return display.Image('screenshot.png')
@@ -576,7 +579,7 @@ class NotebookServer(threading.Thread):
         # directory to a temporary location that
         # will be deleted on exit.
         cfgdir = op.join(tempfile.mkdtemp(prefix='fsleyes-jupyter'), 'config')
-        shutil.copytree(op.join(fsleyes.assetDir, 'assets', 'jupyter'), cfgdir)
+        shutil.copytree(op.join(fsleyes.assetDir, 'jupyter'), cfgdir)
 
         log.debug('Copied notebook configuration to %s', cfgdir)
 
@@ -597,15 +600,7 @@ class NotebookServer(threading.Thread):
         # in a sub-process - we run the server
         # via a wrapper function called nbmain,
         # defined below.
-        #
-        # But with frozen FSLeyes versions, we
-        # can't just call a python interpreter,
-        # so we use a hook in fsleyes.main to
-        # call actions.notebook_main.main.
-        if fslplatform.frozen:
-            cmd = [op.join(op.dirname(sys.executable), 'fsleyes')]
-        else:
-            cmd = [sys.executable, fsleyes_main.__file__]
+        cmd = [sys.executable, fsleyes_main.__file__]
 
         # py2app manipulates the PYTHONPATH,
         # so we pass the cfgdir through as a
