@@ -45,8 +45,6 @@ _WIDGET_SPECS    = td.TypeDict()
 _3D_WIDGET_SPECS = td.TypeDict()
 
 
-
-
 def _merge_dicts(d1, d2):
     d3 = d1.copy()
     d3.update(d2)
@@ -73,9 +71,10 @@ def get3DPropertyList(target):
     return functools.reduce(lambda a, b: a + b, plist)
 
 
-def getWidgetSpecs(target, threedee=False):
+def getWidgetSpecs(target, displayCtx, threedee=False):
 
-    sdicts = _getThing(target, '_initWidgetSpec_', _WIDGET_SPECS, threedee)
+    sdicts = _getThing(target, '_initWidgetSpec_', _WIDGET_SPECS,
+                       displayCtx, threedee)
 
     if sdicts is None:
         return {}
@@ -83,9 +82,10 @@ def getWidgetSpecs(target, threedee=False):
     return functools.reduce(_merge_dicts, sdicts)
 
 
-def get3DWidgetSpecs(target):
+def get3DWidgetSpecs(target, displayCtx):
 
-    sdicts = _getThing(target, '_init3DWidgetSpec_', _3D_WIDGET_SPECS)
+    sdicts = _getThing(target, '_init3DWidgetSpec_', _3D_WIDGET_SPECS,
+                       displayCtx)
 
     if sdicts is None:
         return {}
@@ -149,29 +149,41 @@ def _initPropertyList_Display(threedee):
 
 def _initPropertyList_VolumeOpts(threedee):
     plist = ['custom_volume',
+             'channel',
              'interpolation',
              'custom_cmap',
              'cmapResolution',
              'gamma',
+             'logScale',
              'interpolateCmaps',
              'invert',
              'invertClipping',
              'linkLowRanges',
              'linkHighRanges',
+             'modulateAlpha',
+             'clipImage',
+             'modulateImage',
              'displayRange',
              'clippingRange',
-             'clipImage',
+             'modulateRange',
              'custom_overrideDataRange']
 
     if threedee:
+        plist.remove('modulateAlpha')
         plist.remove('clipImage')
+        plist.remove('modulateImage')
+        plist.remove('modulateRange')
 
     return plist
 
 
+def _initPropertyList_ComplexOpts(threedee):
+    return ['component']
+
+
 def _init3DPropertyList_VolumeOpts():
     return ['blendFactor',
-            'smoothing',
+            'blendByIntensity',
             'numSteps',
             'resolution',
             'custom_clipPlanes']
@@ -194,6 +206,7 @@ def _initPropertyList_VectorOpts(threedee):
             'cmap',
             'clippingRange',
             'modulateRange',
+            'modulateMode',
             'xColour',
             'yColour',
             'zColour',
@@ -204,7 +217,8 @@ def _initPropertyList_VectorOpts(threedee):
 
 
 def _initPropertyList_RGBVectorOpts(threedee):
-    return ['interpolation']
+    return ['interpolation',
+            'unitLength']
 
 
 def _initPropertyList_LineVectorOpts(threedee):
@@ -241,8 +255,11 @@ def _initPropertyList_MeshOpts(threedee):
              'discardClipped',
              'linkLowRanges',
              'linkHighRanges',
+             'modulateAlpha',
+             'modulateData',
              'displayRange',
-             'clippingRange']
+             'clippingRange',
+             'modulateRange']
 
     # Remove outline
     # options for 3D
@@ -254,7 +271,7 @@ def _initPropertyList_MeshOpts(threedee):
 
 
 def _init3DPropertyList_MeshOpts():
-    return ['wireframe']
+    return ['flatShading', 'wireframe']
 
 
 def _initPropertyList_GiftiOpts(threedee):
@@ -291,7 +308,38 @@ def _initPropertyList_SHOpts(threedee):
             'colourMode']
 
 
-def _initWidgetSpec_Display(threedee):
+def _initPropertyList_MIPOpts(threedee):
+    return ['custom_volume',
+            'interpolation',
+            'cmap',
+            'cmapResolution',
+            'gamma',
+            'interpolateCmaps',
+            'invert',
+            'invertClipping',
+            'linkLowRanges',
+            'linkHighRanges',
+            'displayRange',
+            'clippingRange',
+            'window',
+            'minimum',
+            'absolute']
+
+
+def _initPropertyList_VolumeRGBOpts(threedee):
+    return ['custom_volume',
+            'interpolation',
+            'rColour',
+            'gColour',
+            'bColour',
+            'suppressR',
+            'suppressG',
+            'suppressB',
+            'suppressA',
+            'suppressMode']
+
+
+def _initWidgetSpec_Display(displayCtx, threedee):
     return {
         'name'        : props.Widget('name'),
         'overlayType' : props.Widget(
@@ -304,11 +352,10 @@ def _initWidgetSpec_Display(threedee):
     }
 
 
-def _initWidgetSpec_ColourMapOpts(threedee):
+def _initWidgetSpec_ColourMapOpts(displayCtx, threedee):
     return {
-        'custom_cmap'              : _ColourMapOpts_ColourMapWidget,
-        'custom_overrideDataRange' : _VolumeOpts_OverrideDataRangeWidget,
-        'cmap'              : props.Widget(
+        'custom_cmap'     : _ColourMapOpts_ColourMapWidget,
+        'cmap'            : props.Widget(
             'cmap',
             labels=fslcm.getColourMapLabel),
         'useNegativeCmap' : props.Widget('useNegativeCmap'),
@@ -327,11 +374,15 @@ def _initWidgetSpec_ColourMapOpts(threedee):
         'invertClipping'   : props.Widget('invertClipping'),
         'linkLowRanges'    : props.Widget('linkLowRanges'),
         'linkHighRanges'   : props.Widget('linkHighRanges'),
+        'modulateAlpha'    : props.Widget('modulateAlpha'),
+        'logScale'         : props.Widget('logScale'),
         'gamma'            : props.Widget(
             'gamma',
             showLimits=False,
             slider=True,
-            spin=True),
+            spin=True,
+            dependencies=['logScale'],
+            enabledWhen=lambda i, ls : not ls),
         'displayRange'     : props.Widget(
             'displayRange',
             showLimits=False,
@@ -344,17 +395,28 @@ def _initWidgetSpec_ColourMapOpts(threedee):
             slider=True,
             labels=[strings.choices['ColourMapOpts.displayRange.min'],
                     strings.choices['ColourMapOpts.displayRange.max']]),
+        'modulateRange'  : props.Widget(
+            'modulateRange',
+            showLimits=False,
+            slider=True,
+            dependencies=['modulateAlpha'],
+            enabledWhen=lambda i, ma : ma,
+            labels=[strings.choices['ColourMapOpts.displayRange.min'],
+                    strings.choices['ColourMapOpts.displayRange.max']]),
     }
 
 
-def _initWidgetSpec_VolumeOpts(threedee):
+def _initWidgetSpec_VolumeOpts(displayCtx, threedee):
 
     def imageName(img):
         if img is None: return 'None'
-        else:           return img.name
+        else:           return displayCtx.getDisplay(img).name
 
     return {
-        'custom_volume'  : _NiftiOpts_VolumeWidget,
+        'custom_volume'            : _NiftiOpts_VolumeWidget,
+        'custom_overrideDataRange' : _VolumeOpts_OverrideDataRangeWidget,
+
+        'channel'        : props.Widget('channel'),
         'volume'         : props.Widget(
             'volume',
             showLimits=False,
@@ -372,7 +434,9 @@ def _initWidgetSpec_VolumeOpts(threedee):
         'clipImage'      : props.Widget(
             'clipImage',
             labels=imageName),
-        'custom_overrideDataRange' : _VolumeOpts_OverrideDataRangeWidget,
+        'modulateImage'      : props.Widget(
+            'modulateImage',
+            labels=imageName),
         'enableOverrideDataRange'  : props.Widget(
             'enableOverrideDataRange'),
         'overrideDataRange' : props.Widget(
@@ -385,14 +449,14 @@ def _initWidgetSpec_VolumeOpts(threedee):
     }
 
 
-def _init3DWidgetSpec_VolumeOpts():
+def _init3DWidgetSpec_VolumeOpts(displayCtx):
 
     return {
         'numSteps'          : props.Widget('numSteps',
                                            showLimits=False),
         'blendFactor'       : props.Widget('blendFactor',
                                            showLimits=False),
-        'smoothing'         : props.Widget('smoothing',
+        'blendByIntensity'  : props.Widget('blendByIntensity',
                                            showLimits=False),
         'resolution'        : props.Widget('resolution',
                                            showLimits=False),
@@ -413,7 +477,15 @@ def _init3DWidgetSpec_VolumeOpts():
     }
 
 
-def _initWidgetSpec_MaskOpts(threedee):
+def _initWidgetSpec_ComplexOpts(displayCtx, threedee):
+    return {
+        'component' : props.Widget(
+            'component',
+            labels=strings.choices['ComplexOpts.component'])
+    }
+
+
+def _initWidgetSpec_MaskOpts(displayCtx, threedee):
     return {
         'custom_volume'  : _NiftiOpts_VolumeWidget,
         'volume'         : props.Widget(
@@ -438,7 +510,7 @@ def _initWidgetSpec_MaskOpts(threedee):
     }
 
 
-def _initWidgetSpec_LabelOpts(threedee):
+def _initWidgetSpec_LabelOpts(displayCtx, threedee):
     return {
         'lut'          : props.Widget('lut', labels=lambda l: l.name),
         'outline'      : props.Widget('outline'),
@@ -458,10 +530,10 @@ def _initWidgetSpec_LabelOpts(threedee):
     }
 
 
-def _initWidgetSpec_VectorOpts(threedee):
+def _initWidgetSpec_VectorOpts(displayCtx, threedee):
     def imageName(img):
         if img is None: return 'None'
-        else:           return img.name
+        else:           return displayCtx.getDisplay(img).name
 
     return {
         'colourImage'   : props.Widget(
@@ -494,6 +566,11 @@ def _initWidgetSpec_VectorOpts(threedee):
                     strings.choices['VectorOpts.modulateRange.max']],
             dependencies=['colourImage', 'modulateImage'],
             enabledWhen=lambda o, ci, mi: ci is None and mi is not None),
+        'modulateMode' : props.Widget(
+            'modulateMode',
+            labels=strings.choices['VectorOpts.modulateMode'],
+            dependencies=['colourImage', 'modulateImage'],
+            enabledWhen=lambda o, ci, mi: ci is None and mi is not None),
         'xColour'       : props.Widget(
             'xColour',
             dependencies=['colourImage'],
@@ -526,15 +603,16 @@ def _initWidgetSpec_VectorOpts(threedee):
     }
 
 
-def _initWidgetSpec_RGBVectorOpts(threedee):
+def _initWidgetSpec_RGBVectorOpts(displayCtx, threedee):
     return {
         'interpolation' : props.Widget(
             'interpolation',
-            labels=strings.choices['VolumeOpts.interpolation'])
+            labels=strings.choices['VolumeOpts.interpolation']),
+        'unitLength' : props.Widget('unitLength')
     }
 
 
-def _initWidgetSpec_LineVectorOpts(threedee):
+def _initWidgetSpec_LineVectorOpts(displayCtx, threedee):
     return {
         'directed'    : props.Widget('directed'),
         'unitLength'  : props.Widget('unitLength'),
@@ -544,7 +622,7 @@ def _initWidgetSpec_LineVectorOpts(threedee):
     }
 
 
-def _initWidgetSpec_TensorOpts(threedee):
+def _initWidgetSpec_TensorOpts(displayCtx, threedee):
     return {
         'lighting'         : props.Widget('lighting'),
         'orientFlip'       : props.Widget('orientFlip'),
@@ -560,7 +638,7 @@ def _initWidgetSpec_TensorOpts(threedee):
             spin=False),
     }
 
-def _initWidgetSpec_SHOpts(threedee):
+def _initWidgetSpec_SHOpts(displayCtx, threedee):
     return {
         'shResolution'    : props.Widget(
             'shResolution',
@@ -620,11 +698,11 @@ def _initWidgetSpec_SHOpts(threedee):
 
 
 
-def _initWidgetSpec_MeshOpts(threedee):
+def _initWidgetSpec_MeshOpts(displayCtx, threedee):
 
     def imageName(img):
         if img is None: return 'None'
-        else:           return img.name
+        else:           return displayCtx.getDisplay(img).name
 
     def pathName(vdata):
         if vdata is None: return 'None'
@@ -663,6 +741,9 @@ def _initWidgetSpec_MeshOpts(threedee):
         'vertexData'   : props.Widget(
             'vertexData',
             labels=pathName),
+        'modulateData'   : props.Widget(
+            'modulateData',
+            labels=pathName),
         'vertexDataIndex' : props.Widget(
             'vertexDataIndex',
             showLimits=False,
@@ -735,24 +816,84 @@ def _initWidgetSpec_MeshOpts(threedee):
     }
 
 
-def _init3DWidgetSpec_MeshOpts():
+def _init3DWidgetSpec_MeshOpts(displayCtx):
     return {
         'wireframe' : props.Widget('wireframe'),
+        'flatShading' : props.Widget(
+            'flatShading',
+            dependencies=['vertexData'],
+            enabledWhen=lambda o, vd: vd is not None),
     }
 
 
-def _initWidgetSpec_GiftiOpts(threedee):
+def _initWidgetSpec_GiftiOpts(displayCtx, threedee):
     return {}
-def _init3DWidgetSpec_GiftiOpts():
+def _init3DWidgetSpec_GiftiOpts(displayCtx):
     return {}
 
 
-def _initWidgetSpec_FreesurferOpts(threedee):
+def _initWidgetSpec_FreesurferOpts(displayCtx, threedee):
     return {}
-def _init3DWidgetSpec_FreesurferOpts():
+def _init3DWidgetSpec_FreesurferOpts(displayCtx):
     return {}
 
 
+def _initWidgetSpec_MIPOpts(displayCtx, threedee):
+
+    return {
+        'custom_volume' : _NiftiOpts_VolumeWidget,
+        'volume'         : props.Widget(
+            'volume',
+            showLimits=False,
+            enabledWhen=lambda o: o.overlay.ndim > 3,
+            spinWidth=6),
+        'volumeDim'      : props.Widget(
+            'volumeDim',
+            showLimits=False,
+            slider=False,
+            enabledWhen=lambda o: o.overlay.ndim > 4,
+            spinWidth=2),
+        'interpolation'  : props.Widget(
+            'interpolation',
+            labels=strings.choices['VolumeOpts.interpolation']),
+        'window' : props.Widget(
+            'window',
+            showLimits=False,
+            slider=True),
+        'minimum'  : props.Widget('minimum'),
+        'absolute' : props.Widget('absolute'),
+    }
+
+
+def _initWidgetSpec_VolumeRGBOpts(displayCtx, threedee):
+
+    return {
+        'custom_volume'  : _NiftiOpts_VolumeWidget,
+        'volume'         : props.Widget(
+            'volume',
+            showLimits=False,
+            enabledWhen=lambda o: o.overlay.ndim > 3,
+            spinWidth=6),
+        'volumeDim'      : props.Widget(
+            'volumeDim',
+            showLimits=False,
+            slider=False,
+            enabledWhen=lambda o: o.overlay.ndim > 4,
+            spinWidth=2),
+        'interpolation'  : props.Widget(
+            'interpolation',
+            labels=strings.choices['VolumeOpts.interpolation']),
+        'rColour'      : props.Widget('rColour'),
+        'gColour'      : props.Widget('gColour'),
+        'bColour'      : props.Widget('bColour'),
+        'suppressR'    : props.Widget('suppressR'),
+        'suppressG'    : props.Widget('suppressG'),
+        'suppressB'    : props.Widget('suppressB'),
+        'suppressA'    : props.Widget('suppressA'),
+        'suppressMode' : props.Widget('suppressMode'),
+    }
+
+
 def _ColourMapOpts_ColourMapWidget(
         target,
         parent,
@@ -800,10 +941,7 @@ def _ColourMapOpts_ColourMapWidget(
         if bmp is None:
             return
 
-        if fwidgets.wxversion() == fwidgets.WX_PHOENIX:
-            bmp = wx.Bitmap.FromBufferRGBA(w, h, bmp.transpose(1, 0, 2))
-        else:
-            bmp = wx.BitmapFromBufferRGBA( w, h, bmp.transpose(1, 0, 2))
+        bmp = wx.Bitmap.FromBufferRGBA(w, h, bmp.transpose(1, 0, 2))
 
         cbpanel.SetImage(bmp.ConvertToImage())
 
@@ -858,7 +996,7 @@ def _NiftiOpts_VolumeWidget(
                              label='Dim',
                              style=(wx.ALIGN_CENTRE_VERTICAL |
                                     wx.ALIGN_CENTRE_HORIZONTAL))
-
+    dimLabel.SetMinSize(dimLabel.GetBestSize())
     sizer = wx.BoxSizer(wx.HORIZONTAL)
 
     sizer.Add(volume,    flag=wx.EXPAND, proportion=1)
