@@ -12,19 +12,19 @@ views.
 
 import wx
 
-import fsleyes_props          as props
+import fsleyes_props                 as props
 
-import fsleyes.panel          as fslpanel
-import fsleyes.tooltips       as fsltooltips
-import fsleyes.displaycontext as fsldisplay
-import fsleyes.strings        as strings
+import fsleyes.controls.controlpanel as ctrlpanel
+import fsleyes.views.plotpanel       as plotpanel
+import fsleyes.tooltips              as fsltooltips
+import fsleyes.displaycontext        as fsldisplay
+import fsleyes.strings               as strings
 
 
-class PlotControlPanel(fslpanel.FSLeyesSettingsPanel):
+class PlotControlPanel(ctrlpanel.SettingsPanel):
     """The ``PlotControlPanel`` is a *FSLeyes control* panel which allows
     the user to control a :class:`.OverlayPlotPanel`. The ``PlotControlPanel``
-    may be used as is, or may be sub-classed for more customisation.
-
+    is intended to be sub-classed.
 
     Sub-class implementations may:
 
@@ -59,7 +59,22 @@ class PlotControlPanel(fslpanel.FSLeyesSettingsPanel):
     """
 
 
-    def __init__(self, parent, overlayList, displayCtx, frame, plotPanel):
+    @staticmethod
+    def supportedViews():
+        """The ``PlotControlPanel`` is restricted for use with
+        :class:`.OverlayPlotPanel` views. This method may be overridden by
+        sub-classes.
+        """
+        return [plotpanel.OverlayPlotPanel]
+
+
+    @staticmethod
+    def ignoreControl():
+        """The ``PlotControlPanel`` is not intended to be used directly. """
+        return True
+
+
+    def __init__(self, parent, overlayList, displayCtx, plotPanel):
         """Create a ``PlotControlPanel``.
 
         :arg parent:      The :mod:`wx` parent object.
@@ -68,18 +83,16 @@ class PlotControlPanel(fslpanel.FSLeyesSettingsPanel):
 
         :arg displayCtx:  The :class:`.DisplayContext` instance.
 
-        :arg frame:       The :class:`.FSLeyesFrame` instance.
-
         :arg plotPanel:   The :class:`.PlotPanel` associated with this
                           ``PlotControlPanel``.
         """
 
-        fslpanel.FSLeyesSettingsPanel.__init__(self,
-                                               parent,
-                                               overlayList,
-                                               displayCtx,
-                                               frame,
-                                               kbFocus=True)
+        ctrlpanel.SettingsPanel.__init__(self,
+                                         parent,
+                                         overlayList,
+                                         displayCtx,
+                                         plotPanel,
+                                         kbFocus=True)
 
         self.__plotPanel = plotPanel
 
@@ -119,12 +132,13 @@ class PlotControlPanel(fslpanel.FSLeyesSettingsPanel):
                                 self.name,
                                 self.__selectedOverlayChanged)
 
-        plotPanel.addListener('xAutoScale',
-                              self.name,
-                              self.__autoScaleChanged)
-        plotPanel.addListener('yAutoScale',
-                              self.name,
-                              self.__autoScaleChanged)
+        plotCanvas = plotPanel.canvas
+        plotCanvas.addListener('xAutoScale',
+                               self.name,
+                               self.__autoScaleChanged)
+        plotCanvas.addListener('yAutoScale',
+                               self.name,
+                               self.__autoScaleChanged)
 
         # This attribute keeps track of the currently
         # selected overlay, so the widget list group
@@ -139,12 +153,20 @@ class PlotControlPanel(fslpanel.FSLeyesSettingsPanel):
     def destroy(self):
         """Must be called when this ``PlotControlPanel`` is no longer
         needed. Removes some property listeners, and calls the
-        :meth:`.FSLeyesPanel.destroy` method.
+        :meth:`.SettingsPanel.destroy` method.
         """
         self.displayCtx .removeListener('selectedOverlay', self.name)
         self.overlayList.removeListener('overlays',        self.name)
-        self.__plotPanel.removeListener('limits',          self.name)
 
+        # the plot panel may have already
+        # cleared its canvas ref before we
+        # get destroyed
+        canvas = self.__plotPanel.canvas
+        if canvas is not None:
+            canvas.removeListener('xAutoScale', self.name)
+            canvas.removeListener('yAutoScale', self.name)
+
+        self.__plotPanel   = None
         self.__plotWidgets = None
         self.__dsWidgets   = None
 
@@ -152,16 +174,25 @@ class PlotControlPanel(fslpanel.FSLeyesSettingsPanel):
             display = self.displayCtx.getDisplay(self.__selectedOverlay)
             display.removeListener('name', self.name)
 
-        fslpanel.FSLeyesPanel.destroy(self)
+        ctrlpanel.SettingsPanel.destroy(self)
 
 
-    def getPlotPanel(self):
+    @property
+    def plotPanel(self):
         """Returns the :class:`.OverlayPlotPanel` associated with this
         ``PlotControlPanel``.
         """
         return self.__plotPanel
 
 
+    @property
+    def plotCanvas(self):
+        """Returns the :class:`.PlotCanvas` associated with this
+        ``PlotControlPanel``.
+        """
+        return self.__plotPanel.canvas
+
+
     def generateCustomPlotPanelWidgets(self, groupName):
         """May be overridden by sub-classes to add a group of widgets
         controlling :class:`.OverlayPlotPanel` properties.
@@ -192,34 +223,77 @@ class PlotControlPanel(fslpanel.FSLeyesSettingsPanel):
         """
 
         widgetList = self.getWidgetList()
-        plotPanel  = self.__plotPanel
+        plotPanel  = self.plotPanel
+        plotCanvas = self.plotCanvas
         allWidgets = []
 
-        plotProps = ['xLogScale',
-                     'yLogScale',
-                     'smooth',
+        plotProps = ['smooth',
                      'legend',
                      'ticks',
                      'grid',
                      'gridColour',
-                     'bgColour',
-                     'xAutoScale',
-                     'yAutoScale']
+                     'bgColour']
 
         for prop in plotProps:
-            widget = props.makeWidget(widgetList, plotPanel, prop)
+            widget = props.makeWidget(widgetList, plotCanvas, prop)
             allWidgets.append(widget)
             widgetList.AddWidget(
                 widget,
-                displayName=strings.properties[plotPanel, prop],
-                tooltip=fsltooltips.properties[plotPanel, prop],
+                displayName=strings.properties[plotCanvas, prop],
+                tooltip=fsltooltips.properties[plotCanvas, prop],
                 groupName=groupName)
 
-        limits = props.makeListWidgets(widgetList, plotPanel, 'limits')
-        xlims  = wx.BoxSizer(wx.HORIZONTAL)
-        ylims  = wx.BoxSizer(wx.HORIZONTAL)
+        limits     = props.makeListWidgets(widgetList, plotCanvas, 'limits')
+        xlogscale  = props.makeWidget(widgetList, plotCanvas, 'xLogScale')
+        ylogscale  = props.makeWidget(widgetList, plotCanvas, 'yLogScale')
+        xinvert    = props.makeWidget(widgetList, plotCanvas, 'invertX')
+        yinvert    = props.makeWidget(widgetList, plotCanvas, 'invertY')
+        xscale     = props.makeWidget(widgetList, plotCanvas, 'xScale')
+        yscale     = props.makeWidget(widgetList, plotCanvas, 'yScale')
+        xoffset    = props.makeWidget(widgetList, plotCanvas, 'xOffset')
+        yoffset    = props.makeWidget(widgetList, plotCanvas, 'yOffset')
+        xautoscale = props.makeWidget(widgetList, plotCanvas, 'xAutoScale')
+        yautoscale = props.makeWidget(widgetList, plotCanvas, 'yAutoScale')
+        xlabel     = props.makeWidget(widgetList, plotCanvas, 'xlabel')
+        ylabel     = props.makeWidget(widgetList, plotCanvas, 'ylabel')
 
         allWidgets.extend(limits)
+        allWidgets.extend([xlogscale,
+                           ylogscale,
+                           xinvert,
+                           yinvert,
+                           xautoscale,
+                           yautoscale,
+                           xscale,
+                           yscale,
+                           xoffset,
+                           yoffset,
+                           xlabel,
+                           ylabel])
+
+        pairs = [('logscale',  xlogscale,  ylogscale),
+                 ('invert',    xinvert,    yinvert),
+                 ('autoscale', xautoscale, yautoscale),
+                 ('scale',     xscale,     yscale),
+                 ('offset',    xoffset,    yoffset),
+                 ('labels',    xlabel,     ylabel)]
+
+        for key, xwidget, ywidget in pairs:
+
+            sizer = wx.BoxSizer(wx.HORIZONTAL)
+
+            sizer.Add(wx.StaticText(widgetList,
+                                    label=strings.labels[self, 'xlabel']))
+            sizer.Add(xwidget, flag=wx.EXPAND, proportion=1)
+            sizer.Add(wx.StaticText(widgetList,
+                                    label=strings.labels[self, 'ylabel']))
+            sizer.Add(ywidget, flag=wx.EXPAND, proportion=1)
+
+            widgetList.AddWidget(
+                sizer,
+                displayName=strings.labels[self, key],
+                tooltip=fsltooltips.misc[  self, key],
+                groupName=groupName)
 
         # Store refs to the limit widgets
         # so they can be enabled/disabled
@@ -228,24 +302,14 @@ class PlotControlPanel(fslpanel.FSLeyesSettingsPanel):
         self.__xLimitWidgets = [limits[0], limits[1]]
         self.__yLimitWidgets = [limits[2], limits[3]]
 
+        xlims = wx.BoxSizer(wx.HORIZONTAL)
+        ylims = wx.BoxSizer(wx.HORIZONTAL)
+
         xlims.Add(limits[0], flag=wx.EXPAND, proportion=1)
         xlims.Add(limits[1], flag=wx.EXPAND, proportion=1)
         ylims.Add(limits[2], flag=wx.EXPAND, proportion=1)
         ylims.Add(limits[3], flag=wx.EXPAND, proportion=1)
 
-        xlabel = props.makeWidget(widgetList, plotPanel, 'xlabel')
-        ylabel = props.makeWidget(widgetList, plotPanel, 'ylabel')
-        labels = wx.BoxSizer(wx.HORIZONTAL)
-
-        allWidgets.extend([xlabel, ylabel])
-
-        labels.Add(wx.StaticText(widgetList,
-                                 label=strings.labels[self, 'xlabel']))
-        labels.Add(xlabel, flag=wx.EXPAND, proportion=1)
-        labels.Add(wx.StaticText(widgetList,
-                                 label=strings.labels[self, 'ylabel']))
-        labels.Add(ylabel, flag=wx.EXPAND, proportion=1)
-
         widgetList.AddWidget(
             xlims,
             displayName=strings.labels[self, 'xlim'],
@@ -256,11 +320,6 @@ class PlotControlPanel(fslpanel.FSLeyesSettingsPanel):
             displayName=strings.labels[self, 'ylim'],
             tooltip=fsltooltips.misc[  self, 'ylim'],
             groupName=groupName)
-        widgetList.AddWidget(
-            labels,
-            displayName=strings.labels[self, 'labels'],
-            tooltip=fsltooltips.misc[  self, 'labels'],
-            groupName=groupName)
 
         return allWidgets
 
@@ -294,7 +353,7 @@ class PlotControlPanel(fslpanel.FSLeyesSettingsPanel):
         if overlay is None:
             return
 
-        ds = self.__plotPanel.getDataSeries(overlay)
+        ds = self.plotPanel.getDataSeries(overlay)
 
         if ds is None:
             return
@@ -437,6 +496,6 @@ class PlotControlPanel(fslpanel.FSLeyesSettingsPanel):
             return
 
         for l in self.__xLimitWidgets:
-            l.Enable(not self.__plotPanel.xAutoScale)
+            l.Enable(not self.plotCanvas.xAutoScale)
         for l in self.__yLimitWidgets:
-            l.Enable(not self.__plotPanel.yAutoScale)
+            l.Enable(not self.plotCanvas.yAutoScale)
