@@ -9,21 +9,25 @@ control* which allows the user to configure a :class:`.TimeSeriesPanel`.
 """
 
 
-import fsleyes_props               as props
-import fsl.data.featimage          as fslfeatimage
+import wx
 
-import fsleyes.tooltips            as fsltooltips
-import fsleyes.plotting.timeseries as timeseries
-import fsleyes.strings             as strings
-from . import                         plotcontrolpanel
+import fsleyes_props                 as props
+import fsl.data.image                as fslimage
+import fsl.data.featimage            as fslfeatimage
+
+import fsleyes.tooltips              as fsltooltips
+import fsleyes.plotting.timeseries   as timeseries
+import fsleyes.strings               as strings
+import fsleyes.views.timeseriespanel as timeseriespanel
+from . import plotcontrolpanel       as plotctrl
 
 
-class TimeSeriesControlPanel(plotcontrolpanel.PlotControlPanel):
+class TimeSeriesControlPanel(plotctrl.PlotControlPanel):
     """The ``TimeSeriesControlPanel`` is a :class:`.PlotContrlPanel` which
     allows the user to configure a :class:`.TimeSeriesPanel`. It contains
     controls which are linked to the properties of the ``TimeSeriesPanel``,
     (which include properties defined on the :class:`.PlotPanel` base class),
-    and the :class:`.TimeSeries` class.
+    and the :class:`.DataSeries` class.
 
 
     A ``TimeSeriesControlPanel`` looks something like this:
@@ -37,13 +41,13 @@ class TimeSeriesControlPanel(plotcontrolpanel.PlotControlPanel):
     or four sections:
 
      - The *Time series plot settings* section has controls which are linked to
-       properties of the :class:`.TimeSeriesPanel` class.
+       properties of the :class:`.PlotCanvas` class.
 
      - The *General plot settings* section has controls which are linked to
        properties of the :class:`.PlotPanel` base class.
 
      - The *Settings for the current time course* section has controls which
-       are linked to properties of the :class:`.TimeSeries` class. These
+       are linked to properties of the :class:`.DataSeries` class. These
        properties define how the *current* time course is displayed (see the
        :class:`.TimeSeriesPanel` class documentation).
 
@@ -53,17 +57,34 @@ class TimeSeriesControlPanel(plotcontrolpanel.PlotControlPanel):
     """
 
 
+    @staticmethod
+    def supportedViews():
+        """Overrides :meth:`.ControlMixin.supportedViews`. The
+        ``TimeSeriesControlPanel`` is only intended to be added to
+        :class:`.TimeSeriesPanel` views.
+        """
+
+        return [timeseriespanel.TimeSeriesPanel]
+
+
+    @staticmethod
+    def defaultLayout():
+        """Returns a dictionary containing layout settings to be passed to
+        :class:`.ViewPanel.togglePanel`.
+        """
+        return {'location' : wx.RIGHT}
+
+
     def __init__(self, *args, **kwargs):
         """Create a ``TimeSeriesControlPanel``. All arguments are passed
         through to the :meth:`.PlotControlPanel.__init__` method.
         """
 
-        plotcontrolpanel.PlotControlPanel.__init__(self, *args, **kwargs)
+        plotctrl.PlotControlPanel.__init__(self, *args, **kwargs)
 
-        tsPanel = self.getPlotPanel()
-        tsPanel.addListener('plotMelodicICs',
-                            self.name,
-                            self.__plotMelodicICsChanged)
+        self.plotPanel.addListener('plotMelodicICs',
+                                   self.name,
+                                   self.__plotMelodicICsChanged)
 
 
     def destroy(self):
@@ -73,7 +94,7 @@ class TimeSeriesControlPanel(plotcontrolpanel.PlotControlPanel):
         """
         psPanel = self.getPlotPanel()
         psPanel.removeListener('plotMelodicICs', self.name)
-        plotcontrolpanel.PlotControlPanel.destroy(self)
+        plotctrl.PlotControlPanel.destroy(self)
 
 
     def generateCustomPlotPanelWidgets(self, groupName):
@@ -85,7 +106,7 @@ class TimeSeriesControlPanel(plotcontrolpanel.PlotControlPanel):
 
         widgetList = self.getWidgetList()
         allWidgets = []
-        tsPanel    = self.getPlotPanel()
+        tsPanel    = self.plotPanel
         tsProps    = ['plotMode',
                       'usePixdim',
                       'plotMelodicICs']
@@ -109,20 +130,33 @@ class TimeSeriesControlPanel(plotcontrolpanel.PlotControlPanel):
     def generateCustomDataSeriesWidgets(self, ts, groupName):
         """Overrides :meth:`.PlotControlPanel.generateCustomDataSeriesWidgets`.
 
-        If the given :class:`.TimeSeries` is a :class:`.FEATTimeSeries`
-        instance, this method adds some widgets for controlling the
-        FEAT-related settings of the instance.
+        Adds some widgets to the widget list for certain time series types.
+        """
+
+        overlay = ts.overlay
+
+        if (isinstance(overlay, fslfeatimage.FEATImage)    and
+            isinstance(ts,      timeseries.FEATTimeSeries) and
+            overlay.hasStats()):
+            return self.__generateFeatWidgets(ts, groupName)
+
+        elif isinstance(ts, timeseries.ComplexTimeSeries):
+            return self.__generateComplexWidgets(ts, groupName)
+
+        else:
+            return []
+
+
+    def __generateFeatWidgets(self, ts, groupName):
+        """Called by :meth:`generateCustomDataSeriesWidgets`. Generates
+        widgets for :class:`.FEATTimeSeries` options, and adds them
+        to the widget list.
         """
 
         overlay    = ts.overlay
         widgetList = self.getWidgetList()
         allWidgets = []
 
-        if not (isinstance(overlay, fslfeatimage.FEATImage)    and
-                isinstance(ts,      timeseries.FEATTimeSeries) and
-                overlay.hasStats()):
-            return allWidgets
-
         full    = props.makeWidget(     widgetList, ts, 'plotFullModelFit')
         res     = props.makeWidget(     widgetList, ts, 'plotResiduals')
         evs     = props.makeListWidgets(widgetList, ts, 'plotEVs')
@@ -199,11 +233,33 @@ class TimeSeriesControlPanel(plotcontrolpanel.PlotControlPanel):
         return allWidgets
 
 
+    def __generateComplexWidgets(self, ts, groupName):
+        """Called by :meth:`generateCustomDataSeriesWidgets`. Generates
+        widgets for :class:`.ComplexTimeSeries` options, and adds them
+        to the widget list.
+        """
+        widgetList = self.getWidgetList()
+        allWidgets = []
+
+        if isinstance(ts, timeseries.ComplexTimeSeries):
+            for propName in ['plotReal', 'plotImaginary',
+                             'plotMagnitude', 'plotPhase']:
+                widg = props.makeWidget(widgetList, ts, propName)
+                widgetList.AddWidget(
+                    widg,
+                    displayName=strings.properties[ts, propName],
+                    tooltip=fsltooltips.properties[ts, propName],
+                    groupName=groupName)
+                allWidgets.append(widg)
+
+        return allWidgets
+
+
     def __plotMelodicICsChanged(self, *a):
         """Called when the :attr:`.TimeSeriesPanel.plotMelodicICs` property
         changes. If the current overlay is a :class:`.MelodicImage`,
         re-generates the widgets in the *current time course* section, as
-        the :class:`.TimeSeries` instance associated with the overlay may
+        the :class:`.DataSeries` instance associated with the overlay may
         have been re-created.
         """
         self.refreshDataSeriesWidgets()
