@@ -43,7 +43,7 @@ class ColourMapOpts(object):
                  ColourMapOpts.__init__(self)
 
       3. Implement the :meth:`getDataRange` and (if necessary)
-         :meth:`getClippingRange` methods.
+         :meth:`getClippingRange` and :meth:`getModulateRange` methods.
 
       4. Call :meth:`updateDataRange` whenever the data driving the colouring
          changes.
@@ -66,6 +66,7 @@ class ColourMapOpts(object):
        updateDataRange
        getDataRange
        getClippingRange
+       getModulateRange
     """
 
 
@@ -113,6 +114,8 @@ class ColourMapOpts(object):
     """Gamma correction factor - exponentially weights the :attr:`cmap`
     and :attr:`negCmap` towards the low or high ends.
 
+    Ignored if :attr:`logScale` is enabled.
+
     This property takes values between -1 and +1. The exponential weight
     that should actually be used to apply gamma correction should be derived
     as follows:
@@ -126,6 +129,15 @@ class ColourMapOpts(object):
     """
 
 
+    logScale = props.Boolean(default=False)
+    """Applies a logarithmic function to the display range.
+
+    Normally, the display range is linearly mapped to the colour map - if this
+    setting is enabled, the logarithm of the display range is linearly mapped
+    to the colour map.
+    """
+
+
     cmapResolution = props.Int(minval=2, maxval=1024, default=256)
     """Resolution for the colour map, i.e. the number of colours to use. """
 
@@ -176,6 +188,20 @@ class ColourMapOpts(object):
     """
 
 
+    modulateAlpha = props.Boolean(default=False)
+    """If ``True``, the :attr:`.Display.alpha` is modulated by the data.
+    Regions with a value near to the low :attr:`modulateRange` will have an
+    alpha near 0, and regions with a value near to the high
+    :attr:`modulateRange` will have an alpha near 1.
+    """
+
+
+    modulateRange = props.Bounds(ndims=1)
+    """Range used to determine how much to modulate :attr:`.Display.alpha`
+    by, when :attr:`modulateAlpha` is active.
+    """
+
+
     @staticmethod
     def realGamma(gamma):
         """Return the value of ``gamma`` property, scaled appropriately.
@@ -217,30 +243,34 @@ class ColourMapOpts(object):
             name    = self.getColourMapOptsListenerName()
             display = self.display
 
-            display    .addListener('brightness',
-                                    name,
-                                    self.__briconChanged,
-                                    immediate=True)
-            display    .addListener('contrast',
-                                    name,
-                                    self.__briconChanged,
-                                    immediate=True)
-            self       .addListener('displayRange',
-                                    name,
-                                    self.__displayRangeChanged,
-                                    immediate=True)
-            self       .addListener('useNegativeCmap',
-                                    name,
-                                    self.__useNegativeCmapChanged,
-                                    immediate=True)
-            self       .addListener('linkLowRanges',
-                                    name,
-                                    self.__linkLowRangesChanged,
-                                    immediate=True)
-            self       .addListener('linkHighRanges',
-                                    name,
-                                    self.__linkHighRangesChanged,
-                                    immediate=True)
+            display.addListener('brightness',
+                                name,
+                                self.__briconChanged,
+                                immediate=True)
+            display.addListener('contrast',
+                                name,
+                                self.__briconChanged,
+                                immediate=True)
+            self   .addListener('displayRange',
+                                name,
+                                self.__displayRangeChanged,
+                                immediate=True)
+            self   .addListener('useNegativeCmap',
+                                name,
+                                self.__useNegativeCmapChanged,
+                                immediate=True)
+            self   .addListener('linkLowRanges',
+                                name,
+                                self.__linkLowRangesChanged,
+                                immediate=True)
+            self   .addListener('linkHighRanges',
+                                name,
+                                self.__linkHighRangesChanged,
+                                immediate=True)
+            self   .addListener('modulateAlpha',
+                                name,
+                                self.__modulateAlphaChanged,
+                                immediate=True)
 
             # Because displayRange and bri/con are intrinsically
             # linked, it makes no sense to let the user sync/unsync
@@ -272,7 +302,7 @@ class ColourMapOpts(object):
         # to the parent.
         if (not self.__registered) or \
            (not self.isSyncedToParent('displayRange')):
-            self.updateDataRange(False, False)
+            self.updateDataRange(False, False, False)
 
 
     def getColourMapOptsListenerName(self):
@@ -305,6 +335,7 @@ class ColourMapOpts(object):
         self   .removeListener('useNegativeCmap', name)
         self   .removeListener('linkLowRanges',   name)
         self   .removeListener('linkHighRanges',  name)
+        self   .removeListener('modulateAlpha',   name)
 
         self.unbindProps(self   .getSyncPropertyName('displayRange'),
                          display,
@@ -320,7 +351,6 @@ class ColourMapOpts(object):
     def getDataRange(self):
         """Must be overridden by sub-classes. Must return the range of the
         data used for colouring as a ``(min, max)`` tuple.  Note that, even
-
         if there is no effective data range, you should return two different
         values for ``min`` and ``max`` (e.g. ``(0, 1)``), because otherwise
         the relationship between the :attr:`displayRange` and the
@@ -348,18 +378,35 @@ class ColourMapOpts(object):
         return None
 
 
+    def getModulateRange(self):
+        """Can be overridden by sub-classes if necessary. If the modulate
+        range is always the same as the data range, this method does not
+        need to be overridden.
+
+        Otherwise, if the modulate ange differs from the data range (see
+        e.g. the :attr:`.VolumeOpts.modulateImage` property), this method must
+        return the modulate range as a ``(min, max)`` tuple.
+
+        When a sub-class implementation wishes to use the default modulate
+        range/behaviour, it should return the value returned by this
+        base-class implementation.
+        """
+        return None
+
+
     @actions.action
     def resetDisplayRange(self):
-        """Resets the :attr:`displayRange` and :attr:`clippingRange` to their
-        initial values.
+        """Resets the :attr:`displayRange`, :attr:`clippingRange`, and
+         :attr:`modulateRange` to their initial values.
         """
-        self.updateDataRange(True, True)
+        self.updateDataRange(True, True, True)
 
 
-    def updateDataRange(self, resetDR=True, resetCR=True):
+    def updateDataRange(self, resetDR=True, resetCR=True, resetMR=True):
         """Must be called by sub-classes whenever the ranges of the underlying
-        data or clipping values change.  Configures the minimum/maximum bounds
-        of the :attr:`displayRange` and :attr:`clippingRange` properties.
+        data or clipping/modulate values change.  Configures the minimum/
+        maximum bounds of the :attr:`displayRange`, :attr:`clippingRange`, and
+        :attr:`modulateRange` properties.
 
         :arg resetDR: If ``True`` (the default), the :attr:`displayRange`
                       property will be reset to the data range returned
@@ -371,13 +418,19 @@ class ColourMapOpts(object):
                       by :meth:`getClippingRange`. Otherwise the existing
                       value will be preserved.
 
+        :arg resetMR: If ``True`` (the default), the :attr:`modulateRange`
+                      property will be reset to the modulate range returned
+                      by :meth:`getModulateRange`. Otherwise the existing
+                      value will be preserved.
+
         Note that both of these flags will be ignored if the existing low/high
-        :attr:`displayRange`/:attr:`clippingRange` values and limits are equal
-        to each other.
+        :attr:`displayRange`/:attr:`clippingRange`/:attr:`modulateRange`
+        values and limits are equal to each other.
         """
 
         dataMin, dataMax = self.getDataRange()
         clipRange        = self.getClippingRange()
+        modRange         = self.getModulateRange()
 
         absolute = self.useNegativeCmap
         drmin    = dataMin
@@ -390,6 +443,9 @@ class ColourMapOpts(object):
         if clipRange is not None: crmin, crmax = clipRange
         else:                     crmin, crmax = drmin, drmax
 
+        if modRange  is not None: mrmin, mrmax = modRange
+        else:                     mrmin, mrmax = drmin, drmax
+
         # Clipping works on >= and <=, so we add
         # a small offset to the display range limits
         # (which are equal to the clipping limiits)
@@ -407,22 +463,27 @@ class ColourMapOpts(object):
         # in the correct order.
         def doUpdate():
 
-            # If display/clipping limit range
+            # If display/clipping/mod limit range
             # is 0, we assume that they haven't
             # yet been set
             drUnset = (self.displayRange .xmin == self.displayRange .xmax and
                        self.displayRange .xlo  == self.displayRange .xhi)
             crUnset = (self.clippingRange.xmin == self.clippingRange.xmax and
                        self.clippingRange.xlo  == self.clippingRange.xhi)
+            mrUnset = (self.modulateRange.xmin == self.modulateRange.xmax and
+                       self.modulateRange.xlo  == self.modulateRange.xhi)
             crGrow  =  self.clippingRange.xhi  == self.clippingRange.xmax
             drUnset =  resetDR or drUnset
             crUnset =  resetCR or crUnset
+            mrUnset =  resetMR or mrUnset
 
-            log.debug('[{}] Updating range limits [dr: {} - {}, ''cr: '
-                      '{} - {}]'.format(id(self), drmin, drmax, crmin, crmax))
+            log.debug('[%s] Updating range limits [dr: %s - %s, cr: '
+                      '%s - %s, mr: %s - %d]',
+                      id(self), drmin, drmax, crmin, crmax, mrmin, mrmax)
 
             self.displayRange .xlim = drmin, drmax
             self.clippingRange.xlim = crmin, crmax
+            self.modulateRange.xlim = mrmin, mrmax
 
             # If the ranges have not yet been set,
             # initialise them to the min/max.
@@ -430,16 +491,19 @@ class ColourMapOpts(object):
             # was previously equal to the max
             # clipping range, keep that relationship,
             # otherwise high values will be clipped.
-            if drUnset: self.displayRange .x   = drmin + droff, dataMax
+            if drUnset: self.displayRange .x   = drmin + droff, drmax
             if crUnset: self.clippingRange.x   = crmin + croff, crmax
+            if mrUnset: self.modulateRange.x   = mrmin,         mrmax
             if crGrow:  self.clippingRange.xhi = crmax
 
-            # If using absolute range values, the low
-            # display/clipping should be set to 0
+            # If using absolute range values, the
+            # low range values should be set to 0
             if absolute and self.displayRange .xlo < 0:
                 self.displayRange.xlo  = 0
             if absolute and self.clippingRange.xlo < 0:
                 self.clippingRange.xlo = 0
+            if absolute and self.modulateRange.xlo < 0:
+                self.modulateRange.xlo = 0
 
         props.safeCall(doUpdate)
 
@@ -558,7 +622,7 @@ class ColourMapOpts(object):
             self.display.enableProperty('contrast')
 
         if kwa.pop('updateDataRange', True):
-            self.updateDataRange(resetDR=False, resetCR=False)
+            self.updateDataRange(False, False, False)
 
 
     def __linkLowRangesChanged(self, *a):
@@ -601,3 +665,16 @@ class ColourMapOpts(object):
 
         if val:
             cRangePV.set(dRangePV.get())
+
+
+    def __modulateAlphaChanged(self, *a):
+        """Called when the :attr:`modulateAlpha` property changes.
+
+        When ``modulateAlpha`` is active, :attr:`.Display.alpha` is disabled,
+        and vice-versa.
+        """
+
+        if self.modulateAlpha:
+            self.display.disableProperty('alpha')
+        else:
+            self.display.enableProperty('alpha')
