@@ -101,17 +101,8 @@ class OverlayGroup(props.HasProperties):
         :arg overlayList: The :class:`.OverlayList`.
         """
 
-        self.__displayCtx  = displayCtx
-        self.__overlayList = overlayList
-        self.__name        = '{}_{}'.format(type(self).__name__, id(self))
-
-        # This dict is used by the __bindDisplayOpts
-        # method to keep track of which group properties
-        # have already been given a value
-        self.__hasBeenSet  = {}
-
         # Import all of the Display/DisplayOpts
-        # classes into the local namespace
+        # classes into the local namespace.
         from fsleyes.displaycontext import (  # noqa
             Display,
             NiftiOpts,
@@ -125,18 +116,36 @@ class OverlayGroup(props.HasProperties):
             LabelOpts,
             TensorOpts)
 
+        self.__displayCtx  = displayCtx
+        self.__overlayList = overlayList
+        self.__name        = '{}_{}'.format(type(self).__name__, id(self))
+
+        # This dict is used by the __bindDisplayOpts
+        # method to keep track of which group properties
+        # have already been given a value
+        self.__hasBeenSet  = {}
+
+        # Save a ref to each type so we
+        # can look up classes by name.
+        self.__typeMappings = {}
+        for clsName in OverlayGroup._groupBindings.keys():
+            self.__typeMappings[clsName] = locals()[clsName]
+
+        overlayList.addListener('overlays',
+                                self.__name,
+                                self.__overlayListChanged)
+
         # Add all of the properties listed
         # in the _groupBindings dict as
         # properties of this OverlayGroup
         # instance.
         for clsName, propNames in OverlayGroup._groupBindings.items():
 
-            cls = locals()[clsName]
+            cls = self.__typeMappings[clsName]
 
             for propName in propNames:
                 prop = copy.copy(getattr(cls, propName))
                 self.addProperty('{}_{}'.format(clsName, propName), prop)
-
                 self.__hasBeenSet[clsName, propName] = False
 
         # Special case - make sure that the NiftiOpts
@@ -163,6 +172,31 @@ class OverlayGroup(props.HasProperties):
         return '[{}]'.format(', '.join([str(o) for o in self.overlays]))
 
 
+    def __contains__(self, ovl):
+        """Returns ``True`` if ``ovl`` is in this ``OverlayGroup``,
+        :attr:`False` otherwise.
+        """
+        return ovl in self.overlays
+
+
+    def __len__(self):
+        """Returns the number of overlays in this ``OverlayGroup``. """
+        return len(self.overlays)
+
+
+    def destroy(self):
+        """Must be called when this ``OverlayGroup`` is no longer needed.
+        Removes all overlays from the group, and removes property listeners.
+        """
+        for overlay in list(self.overlays):
+            self.removeOverlay(overlay)
+
+        self.__overlayList.removeListener('overlays', self.__name)
+
+        self.__overlayList = None
+        self.__displayCtx  = None
+
+
     def addOverlay(self, overlay):
         """Add an overlay to this ``OverlayGroup``.
 
@@ -172,6 +206,9 @@ class OverlayGroup(props.HasProperties):
         ``OverlayGroup``.
         """
 
+        if overlay in self.overlays:
+            return
+
         self.overlays.append(overlay)
 
         display = self.__displayCtx.getDisplay(overlay)
@@ -191,10 +228,19 @@ class OverlayGroup(props.HasProperties):
     def removeOverlay(self, overlay):
         """Remove the given overlay from this ``OverlayGroup``. """
 
+        if overlay not in self.overlays:
+            return
+
+        from . import InvalidOverlayError
+
         self.overlays.remove(overlay)
 
-        display = self.__displayCtx.getDisplay(overlay)
-        opts    = display.opts
+        try:
+            display = self.__displayCtx.getDisplay(overlay)
+        except InvalidOverlayError:
+            return
+
+        opts = display.opts
 
         log.debug('Removing overlay {} from group {}'.format(
             overlay.name, self.__name))
@@ -204,9 +250,24 @@ class OverlayGroup(props.HasProperties):
 
         display.removeListener('overlayType', self.__name)
 
-        if len(self.overlays) == 0:
-            for key in self.__hasBeenSet.keys():
-                self.__hasBeenSet[key] = False
+        # If this was the last overlay of its type,
+        # make sure to clear the hasBeenSet flag
+        # for all relevent properties; otherwise
+        # the properties of the next overlay of
+        # this type to be added will be clobbered
+        # by the old group values.
+        allopts   = [self.__displayCtx.getOpts(o) for o in self.overlays]
+        bindProps = OverlayGroup._groupBindings.get(opts,
+                                                    allhits=True,
+                                                    bykey=True)
+        for clsName, propNames in bindProps.items():
+
+            cls     = self.__typeMappings[clsName]
+            noftype = len([o for o in allopts if issubclass(type(o), cls)])
+
+            if noftype == 0:
+                for propName in propNames:
+                    self.__hasBeenSet[clsName, propName] = False
 
 
     def __bindDisplayOpts(self, target, unbind=False):
@@ -246,7 +307,6 @@ class OverlayGroup(props.HasProperties):
                 # property values with un-initialised
                 # group property values.
                 if not self.__hasBeenSet[clsName, propName]:
-
                     setattr(self, groupName, getattr(target, propName))
                     self.__hasBeenSet[clsName, propName] = True
 
@@ -263,6 +323,16 @@ class OverlayGroup(props.HasProperties):
                                 unbind=unbind)
 
 
+    def __overlayListChanged(self, *a):
+        """Called when overlays are added/removed to/from the
+        :class:`.OverlayList` . Makes sure that the :attr:`overlays` list for
+        this group does not contain any overlays that have been removed.
+        """
+        for ovl in self.overlays:
+            if ovl not in self.__overlayList:
+                self.removeOverlay(ovl)
+
+
     def __overlayTypeChanged(self, value, valid, display, name):
         """This method is called when the :attr:`.Display.overlayType`
         property for an overlay in the group changes.
