@@ -111,9 +111,6 @@ class GLMesh(globject.GLObject):
         - ``flatColour``:  Colour to use for fragments which are outside
                            of the clipping range.
 
-        - ``lightPos``     Light position in display coordinates.
-
-
      - ``preDraw(GLMesh)``: Prepare for drawing (e.g. load shaders)
 
      - ``draw(GLMesh, glType, vertices, indices=None, normals=None, vdata=None)``:  # noqa
@@ -287,10 +284,6 @@ class GLMesh(globject.GLObject):
         opts   .addListener('flatShading',      name, vertices,    weak=False)
         display.addListener('alpha',            name, refreshCmap, weak=False)
 
-        if self.threedee:
-            canvas.opts.addListener('light',    name, shader, weak=False)
-            canvas.opts.addListener('lightPos', name, shader, weak=False)
-
         # We don't need to listen for
         # brightness or contrast, because
         # they are linked to displayRange.
@@ -327,10 +320,6 @@ class GLMesh(globject.GLObject):
         self.opts   .removeListener('flatShading',      self.name)
         self.display.removeListener('alpha',            self.name)
 
-        if self.threedee:
-            self.canvas.opts.removeListener('light',    self.name)
-            self.canvas.opts.removeListener('lightPos', self.name)
-
 
     def registerLut(self):
         """Registers property listeners with the currently registered
@@ -1069,17 +1058,10 @@ class GLMesh(globject.GLObject):
         """
 
         dopts      = self.opts
-        copts      = self.canvas.opts
-        lightPos   = None
+        canvas     = self.canvas
         flatColour = dopts.getConstantColour()
         useNegCmap = (not dopts.useLut) and dopts.useNegativeCmap
 
-        if self.threedee:
-            lightPos  = np.array(copts.lightPos)
-            lightPos *= (copts.zoom / 100.0)
-        else:
-            lightPos = None
-
         if dopts.useLut:
             delta     = 1.0 / (dopts.lut.max() + 1)
             cmapXform = affine.scaleOffsetXform(delta, 0.5 * delta)
@@ -1105,5 +1087,4 @@ class GLMesh(globject.GLObject):
             cmapXform=cmapXform,
             modScale=modScale,
             modOffset=modOffset,
-            flatColour=flatColour,
-            lightPos=lightPos)
+            flatColour=flatColour)
