@@ -12,18 +12,17 @@ encapsulates the data and logic required to render 2D slice of an
 
 import logging
 
-import numpy                              as np
-import OpenGL.GL                          as gl
+import numpy                     as np
+import OpenGL.GL                 as gl
 
-from   fsl.utils.platform import platform as fslplatform
-import fsl.utils.idle                     as idle
-import fsl.transform.affine               as affine
-import fsleyes.gl                         as fslgl
-import fsleyes.gl.routines                as glroutines
-import fsleyes.gl.shaders.filter          as glfilter
-from . import                                textures
-from . import                                glimageobject
-from . import resources                   as glresources
+import fsl.utils.idle            as idle
+import fsl.transform.affine      as affine
+import fsleyes.gl                as fslgl
+import fsleyes.gl.routines       as glroutines
+import fsleyes.gl.shaders.filter as glfilter
+from . import                       textures
+from . import                       glimageobject
+from . import resources          as glresources
 
 
 log = logging.getLogger(__name__)
@@ -107,6 +106,16 @@ class GLVolume(glimageobject.GLImageObject):
     this clipping image is stored as another :class:`.ImageTexture`.
 
 
+    If the :attr:`.ColourMapOpts.modulateAlpha` setting is active, the opacity
+    of rendered voxels is modulated by the voxel intensity.  If
+    :attr:`.VolumeOpts.modulateImage` is also set, the opacity is modulated by
+    the values in a different image - in this case, this image is stored in
+    another :class:`.ImageTexture`.
+
+    An :class:`.AuxImageTextureManager` is used to manage the clip and modulate
+    textures.
+
+
     **3D rendering**
 
 
@@ -159,8 +168,9 @@ class GLVolume(glimageobject.GLImageObject):
     ``imageTexture``     The :class:`.ImageTexture` which stores the image
                          data.
     ``clipTexture``      The :class:`.ImageTexture` which stores the clip
-                         image data.   If :attr:`.VolumeOpts.clipImage`
-                         is ``None``, this attribute will also be ``None``.
+                         image data.
+    ``modulateTexture``  The :class:`.ImageTexture` which stores the
+                         modulate image data.
     ``colourTexture``    The :class:`.ColourMapTexture` used to store the
                          colour map.
     ``negColourTexture`` The :class:`.ColourMapTexture` used to store the
@@ -216,15 +226,15 @@ class GLVolume(glimageobject.GLImageObject):
         # create this for us.
         self.shader = None
 
-        # References to the clip image and
-        # associated DisplayOpts instance,
-        # if it is set.
-        self.clipImage        = None
-        self.clipOpts         = None
+        # aux texture manager takes care
+        # of clip+modulate textures
+        self.auxmgr = glimageobject.AuxImageTextureManager(
+            self, clip=None, modulate=None)
+        self.auxmgr.registerAuxImage('clip',     self.opts.clipImage)
+        self.auxmgr.registerAuxImage('modulate', self.opts.modulateImage)
 
         # Refs to all of the texture objects.
         self.imageTexture     = None
-        self.clipTexture      = None
         self.colourTexture    = textures.ColourMapTexture(self.texName)
         self.negColourTexture = textures.ColourMapTexture(
             '{}_neg'.format(self.texName))
@@ -246,15 +256,8 @@ class GLVolume(glimageobject.GLImageObject):
         # See that method for details.
         self.__alwaysNotify = False
 
-        # If the VolumeOpts instance has
-        # inherited a clipImage value,
-        # make sure we're registered with it.
-        if self.opts.clipImage is not None:
-            self.registerClipImage()
-
         self.refreshColourTextures()
         self.refreshImageTexture()
-        self.refreshClipTexture()
 
         # Call glvolume_funcs.init when the image
         # and clip textures are ready to be used.
@@ -277,21 +280,17 @@ class GLVolume(glimageobject.GLImageObject):
         and calls :meth:`.GLImageObject.destroy`.
         """
 
-        self.deregisterClipImage()
         self.removeDisplayListeners()
 
         self.imageTexture.deregister(self.name)
         glresources.delete(self.imageTexture.name)
 
-        if self.clipTexture is not None:
-            self.clipTexture.deregister(self.name)
-            glresources.delete(self.clipTexture.name)
-
         self.colourTexture   .destroy()
         self.negColourTexture.destroy()
+        self.auxmgr          .destroy()
 
+        self.auxmgr           = None
         self.imageTexture     = None
-        self.clipTexture      = None
         self.colourTexture    = None
         self.negColourTexture = None
 
@@ -322,12 +321,23 @@ class GLVolume(glimageobject.GLImageObject):
         """
         imageTexReady = (self.imageTexture is not None and
                          self.imageTexture.ready())
+        return imageTexReady and self.auxmgr.texturesReady()
 
-        clipTexReady  = (self.clipImage is None or
-                         (self.clipTexture is not None and
-                          self.clipTexture.ready()))
 
-        return imageTexReady and clipTexReady
+    @property
+    def clipTexture(self):
+        """Returns the :class:`.ImageTexture` associated with the
+        :attr:`.VolumeOpts.clipImage`.
+        """
+        return self.auxmgr.texture('clip')
+
+
+    @property
+    def modulateTexture(self):
+        """Returns the :class:`.ImageTexture` associated with the
+        :attr:`.VolumeOpts.modulateImage`.
+        """
+        return self.auxmgr.texture('modulate')
 
 
     def addDisplayListeners(self):
@@ -350,21 +360,28 @@ class GLVolume(glimageobject.GLImageObject):
         display .addListener('alpha',            name, self._alphaChanged)
         opts    .addListener('displayRange',     name,
                              self._displayRangeChanged)
+        opts    .addListener('modulateRange',    name,
+                             self._modulateRangeChanged)
 
         crPVs[0].addListener(name, self._lowClippingRangeChanged)
         crPVs[1].addListener(name, self._highClippingRangeChanged)
 
         opts    .addListener('clipImage',        name, self._clipImageChanged)
+        opts    .addListener('modulateImage',    name,
+                             self._modulateImageChanged)
         opts    .addListener('invertClipping',   name,
                              self._invertClippingChanged)
         opts    .addListener('cmap',             name, self._cmapChanged)
         opts    .addListener('gamma',            name, self._cmapChanged)
+        opts    .addListener('logScale',         name, self._cmapChanged)
         opts    .addListener('interpolateCmaps', name, self._cmapChanged)
         opts    .addListener('negativeCmap',     name, self._cmapChanged)
         opts    .addListener('cmapResolution',   name, self._cmapChanged)
         opts    .addListener('useNegativeCmap',  name,
                              self._useNegativeCmapChanged)
         opts    .addListener('invert',           name, self._invertChanged)
+        opts    .addListener('modulateAlpha',    name,
+                             self._modulateAlphaChanged)
         opts    .addListener('volume',           name, self._volumeChanged)
         opts    .addListener('channel',          name, self._channelChanged)
         opts    .addListener('interpolation',    name,
@@ -384,7 +401,10 @@ class GLVolume(glimageobject.GLImageObject):
             opts.addListener('numInnerSteps',   name,
                              self._numInnerStepsChanged)
             opts.addListener('resolution',      name,  self._resolutionChanged)
-            opts.addListener('blendFactor',     name, self._blendFactorChanged)
+            opts.addListener('blendFactor',     name,
+                             self._blendPropertiesChanged)
+            opts.addListener('blendByIntensity', name,
+                             self._blendPropertiesChanged)
             opts.addListener('smoothing',       name, self._smoothingChanged)
             opts.addListener('showClipPlanes',  name,
                              self._showClipPlanesChanged)
@@ -433,39 +453,44 @@ class GLVolume(glimageobject.GLImageObject):
         name    = self.name
         crPVs   = opts.getPropVal('clippingRange').getPropertyValueList()
 
-        display .removeListener(          'alpha',                   name)
-        opts    .removeListener(          'displayRange',            name)
+        display .removeListener('alpha',                   name)
+        opts    .removeListener('displayRange',            name)
+        opts    .removeListener('modulateRange',           name)
         crPVs[0].removeListener(name)
         crPVs[1].removeListener(name)
-        opts    .removeListener(          'clipImage',               name)
-        opts    .removeListener(          'invertClipping',          name)
-        opts    .removeListener(          'cmap',                    name)
-        opts    .removeListener(          'gamma',                   name)
-        opts    .removeListener(          'interpolateCmaps',        name)
-        opts    .removeListener(          'negativeCmap',            name)
-        opts    .removeListener(          'useNegativeCmap',         name)
-        opts    .removeListener(          'cmapResolution',          name)
-        opts    .removeListener(          'invert',                  name)
-        opts    .removeListener(          'volume',                  name)
-        opts    .removeListener(          'channel',                 name)
-        opts    .removeListener(          'interpolation',           name)
-        opts    .removeListener(          'transform',               name)
-        opts    .removeListener(          'displayXform',            name)
-        opts    .removeListener(          'enableOverrideDataRange', name)
-        opts    .removeListener(          'overrideDataRange',       name)
+        opts    .removeListener('clipImage',               name)
+        opts    .removeListener('modulateImage',           name)
+        opts    .removeListener('invertClipping',          name)
+        opts    .removeListener('cmap',                    name)
+        opts    .removeListener('gamma',                   name)
+        opts    .removeListener('logScale',                name)
+        opts    .removeListener('interpolateCmaps',        name)
+        opts    .removeListener('negativeCmap',            name)
+        opts    .removeListener('useNegativeCmap',         name)
+        opts    .removeListener('cmapResolution',          name)
+        opts    .removeListener('invert',                  name)
+        opts    .removeListener('modulateAlpha',           name)
+        opts    .removeListener('volume',                  name)
+        opts    .removeListener('channel',                 name)
+        opts    .removeListener('interpolation',           name)
+        opts    .removeListener('transform',               name)
+        opts    .removeListener('displayXform',            name)
+        opts    .removeListener('enableOverrideDataRange', name)
+        opts    .removeListener('overrideDataRange',       name)
 
         if self.threedee:
-            opts.removeListener('numSteps',        name)
-            opts.removeListener('numInnerSteps',   name)
-            opts.removeListener('resolution',      name)
-            opts.removeListener('blendFactor',     name)
-            opts.removeListener('smoothing',       name)
-            opts.removeListener('showClipPlanes',  name)
-            opts.removeListener('numClipPlanes',   name)
-            opts.removeListener('clipMode',        name)
-            opts.removeListener('clipPosition',    name)
-            opts.removeListener('clipAzimuth',     name)
-            opts.removeListener('clipInclination', name)
+            opts.removeListener('numSteps',         name)
+            opts.removeListener('numInnerSteps',    name)
+            opts.removeListener('resolution',       name)
+            opts.removeListener('blendFactor',      name)
+            opts.removeListener('blendByIntensity', name)
+            opts.removeListener('smoothing',        name)
+            opts.removeListener('showClipPlanes',   name)
+            opts.removeListener('numClipPlanes',    name)
+            opts.removeListener('clipMode',         name)
+            opts.removeListener('clipPosition',     name)
+            opts.removeListener('clipAzimuth',      name)
+            opts.removeListener('clipInclination',  name)
 
         if self.__syncListenersRegistered:
             opts.removeSyncChangeListener('volume',  name)
@@ -588,77 +613,19 @@ class GLVolume(glimageobject.GLImageObject):
         self.imageTexture.register(self.name, self.__texturesChanged)
 
 
-    def registerClipImage(self):
-        """Called whenever the :attr:`.VolumeOpts.clipImage` property changes.
-        Adds property listeners to the :class:`.NiftiOpts` instance
-        associated with the new clip image, if necessary.
+    def registerAuxImage(self, which, image, onReady=None):
+        """Calls :meth:`.AuxImageTextureManager.registerAuxImage`, making
+        sure that the texture interpolation is set appropriately.
         """
 
-        clipImage = self.opts.clipImage
+        if self.opts.interpolation == 'none': interp = gl.GL_NEAREST
+        else:                                 interp = gl.GL_LINEAR
 
-        if clipImage is None:
-            return
-
-        clipOpts = self.opts.displayCtx.getOpts(clipImage)
-
-        self.clipImage = clipImage
-        self.clipOpts  = clipOpts
-
-        def updateClipTexture(*a):
-            self.clipTexture.set(volume=clipOpts.index()[3:])
-
-        clipOpts.addListener('volume',
-                             self.name,
-                             updateClipTexture,
-                             weak=False)
-
-
-    def deregisterClipImage(self):
-        """Called whenever the :attr:`.VolumeOpts.clipImage` property changes.
-        Removes property listeners from the :class:`.NiftiOpts` instance
-        associated with the old clip image, if necessary.
-        """
-
-        if self.clipImage is None:
-            return
-
-        self.clipOpts.removeListener('volume', self.name)
-
-        self.clipImage = None
-        self.clipOpts  = None
-
-
-    def refreshClipTexture(self):
-        """Re-creates the :class:`.ImageTexture` used to store the
-        :attr:`.VolumeOpts.clipImage`.
-        """
-        clipImage = self.clipImage
-        opts      = self.opts
-        clipOpts  = self.clipOpts
-
-        texName   = '{}_clip_{}'.format(type(self).__name__, id(clipImage))
-
-        if self.clipTexture is not None:
-            self.clipTexture.deregister(self.name)
-            glresources.delete(self.clipTexture.name)
-            self.clipTexture = None
-
-        if clipImage is None:
-            return None
-
-        if opts.interpolation == 'none': interp = gl.GL_NEAREST
-        else:                            interp = gl.GL_LINEAR
-
-        self.clipTexture = glresources.get(
-            texName,
-            textures.ImageTexture,
-            texName,
-            clipImage,
-            interp=interp,
-            volume=clipOpts.volume,
-            notify=False)
-
-        self.clipTexture.register(self.name, self.__texturesChanged)
+        self.auxmgr.texture(which).deregister(self.name)
+        self.auxmgr.registerAuxImage(which, image, interp=interp)
+        self.auxmgr.texture(which).register(self.name, self.__texturesChanged)
+        if onReady is not None:
+            idle.idleWhen(onReady, self.auxmgr.texturesReady)
 
 
     def refreshColourTextures(self):
@@ -666,17 +633,18 @@ class GLVolume(glimageobject.GLImageObject):
         image voxels.
         """
 
-        display = self.display
-        opts    = self.opts
-        alpha   = display.alpha / 100.0
-        cmap    = opts.cmap
-        interp  = opts.interpolateCmaps
-        res     = opts.cmapResolution
-        gamma   = opts.realGamma(opts.gamma)
-        negCmap = opts.negativeCmap
-        invert  = opts.invert
-        dmin    = opts.displayRange[0]
-        dmax    = opts.displayRange[1]
+        display  = self.display
+        opts     = self.opts
+        alpha    = display.alpha / 100.0
+        cmap     = opts.cmap
+        interp   = opts.interpolateCmaps
+        res      = opts.cmapResolution
+        logScale = opts.logScale
+        gamma    = opts.realGamma(opts.gamma)
+        negCmap  = opts.negativeCmap
+        invert   = opts.invert
+        dmin     = opts.displayRange[0]
+        dmax     = opts.displayRange[1]
 
         if interp: interp = gl.GL_LINEAR
         else:      interp = gl.GL_NEAREST
@@ -686,6 +654,7 @@ class GLVolume(glimageobject.GLImageObject):
                                alpha=alpha,
                                resolution=res,
                                gamma=gamma,
+                               logScale=logScale,
                                interp=interp,
                                displayRange=(dmin, dmax))
 
@@ -708,9 +677,8 @@ class GLVolume(glimageobject.GLImageObject):
         self.imageTexture    .bindTexture(gl.GL_TEXTURE0)
         self.colourTexture   .bindTexture(gl.GL_TEXTURE1)
         self.negColourTexture.bindTexture(gl.GL_TEXTURE2)
-
-        if self.clipTexture is not None:
-            self.clipTexture .bindTexture(gl.GL_TEXTURE3)
+        self.clipTexture     .bindTexture(gl.GL_TEXTURE3)
+        self.modulateTexture .bindTexture(gl.GL_TEXTURE4)
 
         fslgl.glvolume_funcs.preDraw(self, *args, **kwargs)
 
@@ -821,31 +789,55 @@ class GLVolume(glimageobject.GLImageObject):
         self.imageTexture    .unbindTexture()
         self.colourTexture   .unbindTexture()
         self.negColourTexture.unbindTexture()
-
-        if self.clipTexture is not None:
-            self.clipTexture.unbindTexture()
+        self.clipTexture     .unbindTexture()
+        self.modulateTexture .unbindTexture()
 
         fslgl.glvolume_funcs.postDraw(self, *args, **kwargs)
 
 
-    def calculateClipCoordTransform(self):
+    def getAuxTextureXform(self, which):
         """Calculates a transformation matrix which will transform from the
-        image coordinate system into the :attr:`.VolumeOpts.clipImage`
-        coordinate system. If ``clipImage is None``, it will be an identity
-        transform.
+        image coordinate system into the :attr:`.VolumeOpts.clipImage` or
+        :attr:`.VolumeOpts.modulateImage` coordinate system. If the property
+        is ``None``, it will be an identity transform.
 
-        This transform is used by shader programs to find the clip image
+        This transform is used by shader programs to find the auxillary image
         coordinates that correspond with specific image coordinates.
         """
-        if self.opts.clipImage is None:
-            clipCoordXform = np.eye(4)
-        else:
-            clipCoordXform = affine.concat(
-                self.clipOpts.getTransform('display', 'texture'),
-                self.opts    .getTransform('texture', 'display'),
-                self.imageTexture.invTexCoordXform(self.overlay.shape))
+        return affine.concat(
+            self.auxmgr.textureXform(which),
+            # to support 2D image textures
+            self.imageTexture.invTexCoordXform(self.overlay.shape))
 
-        return clipCoordXform
+
+    def getModulateValueXform(self):
+        """Returns an affine transform to normalise alpha modulation values.
+
+        The GL volume shaders need to normalise the modulate value by the
+        modulation range to generate an opacity value. We calculate a suitable
+        scale and offset by buildin an affine transform which transforms voxel
+        values from the image/modulate image texture range to 0/1, where 0
+        corresponds to the low modulate range bound, and 1 to the high
+        modulate range bound. The resulting scale/offset can be used by the
+        shader to convert a modulate value directly into an opacity value.
+        """
+
+        opts = self.opts
+        if opts.modulateImage is None:
+            modXform = self.imageTexture.voxValXform
+        else:
+            modXform = self.modulateTexture.voxValXform
+
+        modlo, modhi = opts.modulateRange
+        modrange     = modhi - modlo
+        if modrange == 0:
+            modXform = np.eye(4)
+        else:
+            modXform = affine.concat(
+                affine.scaleOffsetXform(1 / modrange, -modlo / modrange),
+                modXform)
+
+        return modXform
 
 
     def generateVertices2D(self, zpos, axes, bbox=None):
@@ -921,12 +913,22 @@ class GLVolume(glimageobject.GLImageObject):
         self.updateShaderState(self)
 
 
+    def _modulateRangeChanged(self, *a):
+        """Called when the :attr:`.VolumeOpts.modulateRange` property changes.
+        """
+        self.updateShaderState()
+
+
     def _clipImageChanged(self, *a):
         """Called when the :attr:`.VolumeOpts.clipImage` property changes.
         """
-        self.deregisterClipImage()
-        self.registerClipImage()
-        self.refreshClipTexture()
+        self.registerAuxImage('clip', self.opts.clipImage)
+
+
+    def _modulateImageChanged(self, *a):
+        """Called when the :attr:`.VolumeOpts.modulateImage` property changes.
+        """
+        self.registerAuxImage('modulate', self.opts.modulateImage)
 
 
     def _invertClippingChanged(self, *a):
@@ -956,6 +958,13 @@ class GLVolume(glimageobject.GLImageObject):
         self.notify()
 
 
+    def _modulateAlphaChanged(self, *a):
+        """Called when the :attr:`.VolumeOpts.modulateAlpha` property changes.
+        Calls :meth:`updateShaderState`.
+        """
+        self.updateShaderState()
+
+
     def _enableOverrideDataRangeChanged(self, *a):
         """Called when the :attr:`.VolumeOpts.enableOverrideDataRange` property
         changes. Calls :meth:`_volumeChanged`.
@@ -991,8 +1000,8 @@ class GLVolume(glimageobject.GLImageObject):
                               volRefresh=volRefresh,
                               normaliseRange=normRange)
 
-        if self.clipTexture is not None:
-            self.clipTexture.set(interp=interp)
+        self.clipTexture    .set(interp=interp)
+        self.modulateTexture.set(interp=interp)
 
 
     def _channelChanged(self, *a, **kwa):
@@ -1045,7 +1054,7 @@ class GLVolume(glimageobject.GLImageObject):
         """Called when the :attr:`.Volume3DOpts.numClipPlanes` property
         changes.
         """
-        if float(fslplatform.glVersion) == 1.4:
+        if float(fslgl.GL_COMPATIBILITY) == 1.4:
             fslgl.glvolume_funcs.compileShaders(self)
         self.updateShaderState(alwaysNotify=True)
 
@@ -1054,7 +1063,7 @@ class GLVolume(glimageobject.GLImageObject):
         """Called when the :attr:`.Volume3DOpts.clipMode` property
         changes.
         """
-        if float(fslplatform.glVersion) == 1.4:
+        if float(fslgl.GL_COMPATIBILITY) == 1.4:
             fslgl.glvolume_funcs.compileShaders(self)
         self.updateShaderState(alwaysNotify=True)
 
@@ -1075,7 +1084,7 @@ class GLVolume(glimageobject.GLImageObject):
         self.updateShaderState(alwaysNotify=True)
 
 
-    def _blendFactorChanged(self, *a):
+    def _blendPropertiesChanged(self, *a):
         """Called when the :attr:`.Volume3DOpts.showClipPlanes` property
         changes.
         """
