@@ -33,10 +33,6 @@ This logic is encapsulated in four classes:
    create and change the ``Profile`` instance currently in use.
 
 
-The :mod:`.profilemap` module contains mappings between ``ViewPanel`` types,
-and their corresponding ``Profile`` types.
-
-
 The ``profiles`` package is also home to the :mod:`.shortcuts` module, which
 defines global *FSLeyes* keyboard shortcuts.
 """
@@ -44,58 +40,63 @@ defines global *FSLeyes* keyboard shortcuts.
 
 import logging
 import inspect
+import collections
 
 import wx
 
 import matplotlib.backend_bases as mplbackend
 
-from   fsl.utils.platform import platform as fslplatform
-import fsleyes_props                      as props
-import fsleyes.actions                    as actions
+import fsl.utils.notifier   as notifier
+import fsleyes_widgets      as fwidgets
+import fsleyes_props        as props
+import fsleyes.actions      as actions
 
 
 log = logging.getLogger(__name__)
 
 
-class ProfileManager(object):
+class ProfileManager(notifier.Notifier):
     """Manages creation/registration/de-registration of :class:`Profile`
     instances for a :class:`.ViewPanel` instance.
 
     A :class:`ProfileManager` instance is created and used by every
-    :class:`.ViewPanel` instance. The :mod:`.profilemap` module defines the
-    :class:`Profile` types which should used for specific :class:`.ViewPanel`
-    types.
+    :class:`.ViewPanel` instance. The ``ProfileManager`` manages a stack of
+    :class:`.Profile` instances - profiles can be activated and deactivated
+    via the :meth:`activateProfile` and :meth:`deactivateProfile` methods.
+
+    The ``ProfileManager`` uses the :class:`.Notifier` interface to notify any
+    interested listeners when the current profile is changed, using the topic
+    name ``'profile'``. Registered listeners are passed a tuple containing the
+    types (:class:`.Profile` sub-classes) of the de-registered and newly
+    registered profiles.
     """
 
 
-    def __init__(self, viewPanel, overlayList, displayCtx):
+    def __init__(self, viewPanel, overlayList, displayCtx, maxprofiles):
         """Create a :class:`ProfileManager`.
 
-        :arg viewPanel:   The :class:`.ViewPanel` instance which this
-                          :class:`ProfileManager` is to manage.
+        :arg viewPanel:      The :class:`.ViewPanel` instance which this
+                             :class:`ProfileManager` is to manage.
 
-        :arg overlayList: The :class:`.OverlayList` instance containing the
-                          overlays that are being displayed.
+        :arg overlayList:    The :class:`.OverlayList` instance containing the
+                             overlays that are being displayed.
 
-        :arg displayCtx:  The :class:`.DisplayContext` instance which defines
-                          how overlays are being displayed.
+        :arg displayCtx:     The :class:`.DisplayContext` instance which
+                             defines how overlays are being displayed.
+
+        :arg maxprofiles:    Maximum numbe of profiles which can be in the
+                             profile stack.
         """
-        from . import profilemap
 
-        self.__viewPanel      = viewPanel
-        self.__viewCls        = viewPanel.__class__
-        self.__overlayList    = overlayList
-        self.__displayCtx     = displayCtx
-        self.__currentProfile = None
+        self.__viewPanel   = viewPanel
+        self.__viewCls     = viewPanel.__class__
+        self.__overlayList = overlayList
+        self.__displayCtx  = displayCtx
+        self.__maxprofiles = maxprofiles
 
-        profileProp = viewPanel.getProp('profile')
-        profilez    = profilemap.profiles.get(viewPanel.__class__, [])
-
-        for profile in profilez:
-            profileProp.addChoice(profile, instance=viewPanel)
-
-        if len(profilez) > 0:
-            viewPanel.profile = profilez[0]
+        # a stack of profiles - [0] is the
+        # default, and [-1] is currently active
+        self.__profileStack = []
 
 
     def destroy(self):
@@ -103,55 +104,98 @@ class ProfileManager(object):
         it is about to be destroyed (or when it no longer needs a
         ``ProfileManager``).
 
-        This method destroys the current :class:`Profile` (if any), and
-        clears some internal object references to avoid memory leaks.
+        This method destroys all class:`Profile` instances, and clears some
+        internal object references to avoid memory leaks.
         """
-        if self.__currentProfile is not None:
-            self.__currentProfile.deregister()
-            self.__currentProfile.destroy()
 
-        self.__currentProfile    = None
-        self.__viewPanel         = None
-        self.__overlayList       = None
-        self.__overlaydisplayCtx = None
+        if self.numProfiles() > 0:
+            self.__profileStack[-1].deregister()
+
+        for prof in self.__profileStack:
+            prof.destroy()
+
+        self.__profileStack = []
+        self.__viewPanel    = None
+        self.__overlayList  = None
+        self.__displayCtx   = None
+
+
+    def numProfiles(self):
+        """Returns the number of profiles that currently exist. """
+        return len(self.__profileStack)
 
 
     def getCurrentProfile(self):
         """Returns the :class:`Profile` instance currently in use."""
-        return self.__currentProfile
+
+        if self.numProfiles() > 0:
+            return self.__profileStack[-1]
+        else:
+            return None
 
 
-    def changeProfile(self, profile):
-        """Deregisters the current :class:`Profile` instance (if necessary),
-        and creates a new one corresponding to the named profile.
+    def deactivateProfile(self, notify=True):
+        """Deactivates and destroys the current profile, and re-activates
+        the previous one.
+
+        :arg notify: If ``True`` (the default), a notification is emitted
+                     via the :class:`.Notifier` interface.
+        :returns:    A reference to the re-activated profile.
         """
 
-        from . import profilemap
+        oldprof = self.__profileStack.pop()
 
-        profileCls = profilemap.profileHandlers[self.__viewCls, profile]
+        log.debug('Deregistering %s profile from view %s',
+                  type(oldprof).__name__, self.__viewCls.__name__)
 
-        # the current profile is the requested profile
-        if (self.__currentProfile is not None) and \
-           (self.__currentProfile.__class__ is profileCls):
-            return
+        oldprof.deregister()
+        oldprof.destroy()
 
-        if self.__currentProfile is not None:
-            log.debug('Deregistering {} profile from {}'.format(
-                self.__currentProfile.__class__.__name__,
-                self.__viewCls.__name__))
-            self.__currentProfile.deregister()
-            self.__currentProfile.destroy()
-            self.__currentProfile = None
+        if self.numProfiles() > 0:
+            self.__profileStack[-1].register()
 
-        self.__currentProfile = profileCls(self.__viewPanel,
-                                           self.__overlayList,
-                                           self.__displayCtx)
+        newprof = self.getCurrentProfile()
 
-        log.debug('Registering {} profile with {}'.format(
-            self.__currentProfile.__class__.__name__,
-            self.__viewCls.__name__))
+        if notify:
+            if newprof is None: value = (type(oldprof), None)
+            else:               value = (type(oldprof), type(newprof))
+            self.notify(topic='profile', value=value)
 
-        self.__currentProfile.register()
+        return newprof
+
+
+    def activateProfile(self, profileCls, notify=True):
+        """Deregisters the current :class:`Profile` instance, and creates and
+        registers a new instance of type ``profileCls``.
+
+        :arg notify: If ``True`` (the default), a notification is emitted
+                     via the :class:`.Notifier` interface.
+        :returns:    A reference to the newly registered profile.
+        """
+
+        if self.numProfiles() == self.__maxprofiles:
+            raise RuntimeError('Cannot create any more profiles!')
+
+        if self.numProfiles() > 0:
+            self.__profileStack[-1].deregister()
+
+        log.debug('Creating and registering profile %s with view %s',
+                  profileCls.__name__, type(self.__viewPanel).__name__)
+
+        oldprof = self.getCurrentProfile()
+        newprof = profileCls(self.__viewPanel,
+                             self.__overlayList,
+                             self.__displayCtx)
+
+        self.__profileStack.append(newprof)
+        self.__profileStack[-1].register()
+
+        if notify:
+            if oldprof is None: value = (None,          type(newprof))
+            else:               value = (type(oldprof), type(newprof))
+            self.notify(topic='profile', value=value)
+
+        return newprof
 
 
 class Profile(props.SyncableHasProperties, actions.ActionProvider):
@@ -172,10 +216,13 @@ class Profile(props.SyncableHasProperties, actions.ActionProvider):
     The ``Profile`` class currently only supports :class:`.CanvasPanel` and
     :class:`.PlotPanel` views. ``Profile`` instances use a
     :class:`.CanvasPanelEventManager` instance to manage GUI events on
-    :class:`.CanvasPanel` instances, or a:class:`.PlotPanelEventMAnager`
+    :class:`.CanvasPanel` instances, or a:class:`.PlotPanelEventManager`
     to manage GUI events on ``matplotlib Canvas`` objects.
 
 
+    **Receiving events**
+
+
     In order to receive mouse or keyboard events, subclasses simply need to
     implement methods which handle the events of interest for the relevant
     mode, and name them appropriately. The name of a method handler must be
@@ -225,10 +272,69 @@ class Profile(props.SyncableHasProperties, actions.ActionProvider):
     ``nav`` mode, this method will be called on left mouse clicks.
 
 
-    These methods should return ``True`` to indicate that the event was
-    handled, or ``False``  if the event was not handled. This is particularly
+    **Handler methods**
+
+
+    The parameters that are passed to these methods differs slightly depending
+    on the type of event:
+
+     - All mouse events, with the exception of ``MouseWheel`` must have
+       the following signature::
+
+        def _[modeName]Mode[eventName](ev, canvas, mouseLoc, canvasLoc)
+
+       where:
+
+         - ``ev`` is the ``wx.Event`` object
+         - ``canvas`` is the source canvas,
+         - ``mouseLoc`` is the ``(x, y)`` mouse coordinates,
+         - ``canvasLoc`` is the coordinates in the display/data coordinate
+           system.
+
+     - The ``MouseWheel`` handler must have the following signature::
+
+        def _[modeName]ModeMouseWheel(ev, canvas, wheel, mouseLoc, canvasLoc)
+
+       where ``wheel`` is a positive or negative number indicating how much
+       the mouse wheel was moved.
+
+     - ``Char`` events must have the following signature::
+
+        def _[modeName]ModeChar(ev, canvas, key)
+
+       where ``key`` is the key code of the key that was pressed.
+
+     - Pick event handlers (only on :class:`.PlotPanel` views) must have
+       the following signature::
+
+        def _[modeName]Mode[eventType](ev, canvas, artist, mouseLoc, canvasLoc)
+
+       where ``artist`` is the ``matplotlib`` artist that was picked.
+
+    All handler methods should return ``True`` to indicate that the event was
+    handled, or ``False`` if the event was not handled. This is particularly
     important for ``Char`` event handlers - we don't want ``Profile``
-    sub-classes to be eating global keyboard shortcuts.
+    sub-classes to be eating global keyboard shortcuts. A return value of
+    ``None`` is interpreted as ``True``. If a handler returns ``False``, and
+    a fallback handler is defined (see below), then that fallback handler will
+    be called.
+
+
+    **Extra handlers**
+
+
+    Additional handlers can be registered for any event type via the
+    :meth:`registerHandler` method. These handlers do not have to be methods
+    of the ``Profile`` sub-class, and will be called for every occurrence of
+    the event, regardless of the current mode. These handlers will be called
+    after the standard handler method.
+
+
+    When an extra handler is no longer needed, it must be removed via the
+    :meth:`deregisterHandler` method.
+
+
+    **Pre- and post- methods**
 
 
     A couple of other methods may be defined which, if they are present, will
@@ -239,24 +345,36 @@ class Profile(props.SyncableHasProperties, actions.ActionProvider):
 
     The ``_preEvent`` method will get called just before an event is passed
     to the handler. Likewise, the ``_postEvent`` method will get called
-    just after the handler has been called. If a handler for a particular
-    event is not defined, neither of these methods will be called.
+    just after the handler has been called. If no handlers for a particular
+    event are defined, neither of these methods will be called.
 
 
-    The :mod:`.profilemap` module contains a ``tempModeMap`` which, for each
-    profile and each mode, defines a keyboard modifier which may be used to
-    temporarily redirect mouse/keyboard events to the handlers for a different
-    mode. For example, if while in ``nav`` mode, you would like the user to be
-    able to switch to ``zoom`` mode with the control key, you can add a
-    temporary mode map in the ``tempModeMap``.
+    **Temporary, alternate and fallback handlers**
 
 
-    The :mod:`.profilemap` module contains another dictionary, called the
-    ``altHandlerMap``. This dictionary allows you to re-use event handlers
-    that have been defined for one mode in another mode. For example, if you
-    would like right clicks in ``zoom`` mode to behave like left clicks in
-    ``nav`` mode, you can set up such a mapping using the
-    ``altHandlerMap`` dictionary.
+    The :meth:`tempModes` function may be overridden to define a keyboard
+    modifier which may be used to temporarily redirect mouse/keyboard events
+    to the handlers for a different mode. For example, if while in ``nav``
+    mode, you would like the user to be able to switch to ``zoom`` mode with
+    the control key, you can add a temporary mode map in the dictionary
+    returned by ``tempModes``. Additional temporary modes can be added via the
+    :meth:`addTempMode` method.
+
+    The :meth:`altHandlers` function may be overridden to allow the re-use of
+    event handlers that have been defined for one mode in another mode. For
+    example, if you would like right clicks in ``zoom`` mode to behave like
+    left clicks in ``nav`` mode, you can return such a mapping from the
+    ``altHandler`` function. Additional alternate handlers can be added
+    via the :meth:`addAltHandler` method.
+
+    The :meth:`fallbackHandlers` function may be overridden to define fallback
+    handlers - if the default handler for a specific mode/event type returns a
+    value of ``False``, the event will be forwarded to the fallback handler
+    instead. Additional fallback handlers can be added via the
+    :meth:`addFallbackHandler` method.
+
+
+    **Actions and attributes**
 
 
     As the ``Profile`` class derives from the :class:`.ActionProvider`
@@ -268,12 +386,12 @@ class Profile(props.SyncableHasProperties, actions.ActionProvider):
     The following instance attributes are present on a ``Profile`` instance,
     intended to be accessed by sub-classes:
 
-    ================ =======================================================
-    ``_viewPanel``   The :class:`ViewPanel` which is using this ``Profile``.
-    ``_overlayList`` A :class:`.OverlayList` instance.
-    ``_displayCtx``  A :class:`.DisplayContext` instance.
-    ``_name``        A unique name for this ``Profile`` instance.
-    ================ =======================================================
+    =============== =======================================================
+    ``viewPanel``   The :class:`ViewPanel` which is using this ``Profile``.
+    ``overlayList`` A :class:`.OverlayList` instance.
+    ``displayCtx``  A :class:`.DisplayContext` instance.
+    ``name``        A unique name for this ``Profile`` instance.
+    =============== =======================================================
     """
 
 
@@ -283,6 +401,90 @@ class Profile(props.SyncableHasProperties, actions.ActionProvider):
     """
 
 
+    @staticmethod
+    def supportedView():
+        """Returns the :class:`.ViewPanel` type that is supported by this
+        ``Profile``. Must be implemented by sub-classes.
+        """
+        raise NotImplementedError()
+
+
+    @staticmethod
+    def tempModes():
+        """May be overridden by sub-classes.  Should return a dictionary
+        defining temporary modes which, when in a given mode, can be accessed
+        with a keyboard modifer (e.g. Control, Shift, etc). For example, a
+        temporary mode map of::
+
+            ('view', wx.WXK_SHIFT) : 'zoom'
+
+        states that when the ``Profile`` is in ``'view'`` mode, and the shift
+        key is held down, the ``Profile`` should temporarily switch to
+        ``'zoom'`` mode.
+
+        For multi-key combinations, the modifier key IDs must be provided as a
+        tuple, in alphabetical order. For example, to specify shift+ctrl, the
+        tuple must be (wx.WXK_CTRL, wx.WXK_SHIFT)
+
+        Important: Any temporary modes which use CTRL, ALT, or CTRL+ALT must
+        not handle character events, as these modifiers are reserved for
+        global shortcuts.
+
+        Temporary modes honour the ``Profile`` class hierarchy, so if you
+        sub-class an existing ``Profile`` class, your class will inherit all
+        of the temporary modes defined on the base class.
+        """
+        return None
+
+
+    @staticmethod
+    def altHandlers():
+        """May be overridden by sub-classes. Should return a dictionary
+        defining alternate handlers for a given mode and event type. Entries
+        in this dictionary allow a :class:`.Profile` sub-class to define a
+        handler for a single mode and event type, but to re-use that handler
+        for other modes and event types. For example, the following alternate
+        handler mapping::
+
+            ('zoom', 'MiddleMouseDrag') : ('pan',  'LeftMouseDrag')
+
+        states that when the ``Profile`` is in ``'zoom'`` mode, and a
+        ``MiddleMouseDrag`` event occurs, the ``LeftMouseDrag`` handler for
+        the ``'pan'`` mode should be called.
+
+        Alternate handlers honour the ``Profile`` class hierarchy, so if you
+        sub-class an existing ``Profile`` class, your class will inherit all
+        of the alternate handlers defined on the base class.
+
+        .. note:: Event bindings defined by ``altHandlers`` take precdence
+                  over the event bindings defined in the :class:`.Profile`
+                  sub-class. So you can use the ``altHandlers`` to override
+                  the default behaviour of a ``Profile``.
+        """
+        return None
+
+
+    @staticmethod
+    def fallbackHandlers():
+        """May be overridden by sub-classes. Should return a dictionary
+        defining handlers for a given mode and event type which will be called
+        if the handler for that mode/event type returns a value of ``False``,
+        indicating that it has not been handled. For example, the
+        following fallback handler mapping::
+
+            (('pick', 'LeftMouseDown'), ('nav', 'LeftMouseDown')),
+
+        states that when the profile is in ``'pick'`` mode, and the
+        ``LeftMouseDown`` handler for ``'pick'`` mode returns ``False``, the
+        ``LeftMouseDown`` handler for ``'nav'`` mode will be called.
+
+        Fallback handlers honour the ``Profile`` class hierarchy, so if you
+        sub-class an existing ``Profile`` class, your class will inherit all
+        of the fallback handlers defined on the base class.
+        """
+        return None
+
+
     def __init__(self,
                  viewPanel,
                  overlayList,
@@ -305,13 +507,15 @@ class Profile(props.SyncableHasProperties, actions.ActionProvider):
                           options on the :attr:`mode` property.
         """
 
-        actions.ActionProvider     .__init__(self)
+        if not isinstance(viewPanel, self.supportedView()):
+            raise ValueError('Unsupported view: {}'.format(
+                type(viewPanel).__name__))
+
+        actions.ActionProvider     .__init__(self, overlayList, displayCtx)
         props.SyncableHasProperties.__init__(self)
 
-        self._viewPanel   = viewPanel
-        self._overlayList = overlayList
-        self._displayCtx  = displayCtx
-        self._name        = '{}_{}'.format(self.__class__.__name__, id(self))
+        self.__viewPanel = viewPanel
+        self.__name      = '{}_{}'.format(self.__class__.__name__, id(self))
 
         import fsleyes.views.canvaspanel as canvaspanel
         import fsleyes.views.plotpanel   as plotpanel
@@ -326,14 +530,23 @@ class Profile(props.SyncableHasProperties, actions.ActionProvider):
             raise ValueError('Unrecognised view panel type: {}'.format(
                 type(viewPanel).__name__))
 
-        # Maps which define temporarymodes/alternate
-        # handlers when keyboard modifiers are used,
-        # or when a handler for a particular event
-        # is not defined
-        self.__tempModeMap   = {}
-        self.__altHandlerMap = {}
+        # Maps which define temporary modes and
+        # alternate/fallback handlers when keyboard
+        # modifiers are used, when a handler for
+        # a particular event is not defined, or
+        # when a handler indicates that the event
+        # has not been handled.
+        self.__tempModeMap        = {}
+        self.__altHandlerMap      = {}
+        self.__fallbackHandlerMap = {}
+
+        # Extra handlers - for each event type,
+        # a dictionary of { name : handler }
+        # mappings which will be called after
+        # the profile handler has been called
+        # for a given event.
+        self.__extraHandlers = collections.defaultdict(collections.OrderedDict)
 
-        # some attributes to keep track
         # of mouse/canvas event locations
         self.__lastCanvas       = None
         self.__lastMousePos     = None
@@ -384,17 +597,16 @@ class Profile(props.SyncableHasProperties, actions.ActionProvider):
             self.mode = modes[0]
 
         # Configure temporary modes and alternate
-        # event handlers - see the profilemap
-        # module
-        from . import profilemap
-
-        # We reverse the mro, so that the
-        # modes/handlers defined on this
+        # event handlers. We reverse the mro, so
+        # that the modes/handlers defined on this
         # class take precedence.
-        for cls in reversed(inspect.getmro(self.__class__)):
+        mro = list(reversed(inspect.getmro(self.__class__)))
+        mro = mro[mro.index(Profile):]
+        for cls in mro:
 
-            tempModes   = profilemap.tempModeMap  .get(cls, {})
-            altHandlers = profilemap.altHandlerMap.get(cls, {})
+            tempModes   = cls.tempModes()        or {}
+            altHandlers = cls.altHandlers()      or {}
+            fbHandlers  = cls.fallbackHandlers() or {}
 
             for (mode, keymod), tempMode in tempModes.items():
                 self.addTempMode(mode, keymod, tempMode)
@@ -402,6 +614,9 @@ class Profile(props.SyncableHasProperties, actions.ActionProvider):
             for (mode, handler), (altMode, altHandler) in altHandlers.items():
                 self.addAltHandler(mode, handler, altMode, altHandler)
 
+            for (mode, handler), (fbMode, fbHandler) in fbHandlers.items():
+                self.addFallbackHandler(mode, handler, fbMode, fbHandler)
+
         # The __onEvent method delegates all
         # events based on this dictionary
         self.__eventMap = {
@@ -435,10 +650,22 @@ class Profile(props.SyncableHasProperties, actions.ActionProvider):
         list, and calls :meth:`.ActionProvider.destroy`.
         """
         actions.ActionProvider.destroy(self)
-        self.__lastCanvas = None
-        self._viewPanel   = None
-        self._overlayList = None
-        self._displayCtx  = None
+        self.__lastCanvas    = None
+        self.__viewPanel     = None
+        self.__extraHandlers = None
+
+
+    @property
+    def name(self):
+        """Returns the name of this ``Profile``. """
+        return self.__name
+
+
+    @property
+    def viewPanel(self):
+        """Returns the :class:`.ViewPanel` associated with this ``Profile``.
+        """
+        return self.__viewPanel
 
 
     def getEventTargets(self):
@@ -515,7 +742,7 @@ class Profile(props.SyncableHasProperties, actions.ActionProvider):
 
     def addTempMode(self, mode, modifier, tempMode):
         """Add a temporary mode to this ``Profile``, in addition to those
-        defined in the :attr:`.profilemap.tempModeMap` dictionary.
+        defined by the :meth:`tempModes` function.
 
         :arg mode:     The mode to change from.
 
@@ -530,8 +757,7 @@ class Profile(props.SyncableHasProperties, actions.ActionProvider):
 
     def addAltHandler(self, mode, event, altMode, altEvent):
         """Add an alternate handler to this ``Profile``, in addition to
-        those already defined in the :attr:`.profilemap.altHandleMap`
-        dictionary.
+        those already defined by the :meth:`altHandlers` function.
 
         :arg mode:     The source mode.
 
@@ -544,6 +770,21 @@ class Profile(props.SyncableHasProperties, actions.ActionProvider):
         self.__altHandlerMap[mode, event] = (altMode, altEvent)
 
 
+    def addFallbackHandler(self, mode, event, fbMode, fbEvent):
+        """Add a fallback handler to this ``Profile``, in addition to
+        those already defined by the :meth:`fallbackHandlers` function.
+
+        :arg mode:    The source mode.
+
+        :arg event:   Name of the event to handle (e.g. ``LeftMouseDown``).
+
+        :arg fbMode:  The mode for which the handler is defined.
+
+        :arg fbEvent: The event name for which the handler is defined.
+        """
+        self.__fallbackHandlerMap[mode, event] = (fbMode, fbEvent)
+
+
     def register(self):
         """This method must be called to register this ``Profile``
         instance as the target for mouse/keyboard events. This method
@@ -560,12 +801,48 @@ class Profile(props.SyncableHasProperties, actions.ActionProvider):
         receiving mouse/keybouard events. This method is called by the
         :class:`ProfileManager`.
 
+
         Subclasses may override this method to performa any initialisation,
         but must make sure to call this implementation.
         """
         self.__evtManager.deregister()
 
 
+    def registerHandler(self, event, name, handler):
+        """Add an extra handler for the specified event.
+
+        When the event occurs, The ``handler`` function will be called after
+        the default handler, provided by the  ``Profie`` sub-class, is called.
+
+        :arg event:   The event type (e.g. ``LeftMouseDown``).
+
+        :arg name:    A unique name for the handler. A ``KeyError`` will be
+                      raised if a handler with ``name`` is already registered.
+
+        :arg handler: Function to call when the event occurs. See the class
+                      documentation for details on the required signature.
+        """
+
+        if name in self.__extraHandlers[event]:
+            raise KeyError('A handler with name "{}" is '
+                           'already registered'.format(name))
+
+        self.__extraHandlers[event][name] = handler
+
+
+    def deregisterHandler(self, event, name):
+        """Remove an extra handler from the specified event, that was previously
+        added via :meth:`registerHandler`
+
+        :arg event:   The event type (e.g. ``LeftMouseDown``).
+
+        :arg name:    A unique name for the handler. A ``KeyError`` will be
+                      raised if a handler with ``name`` is already registered.
+        """
+        if self.__extraHandlers is not None:
+            self.__extraHandlers[event].pop(name)
+
+
     def handleEvent(self, ev):
         """Called by the event manager when any event occurs on any of
         the :class:`.ViewPanel` targets. Delegates the event to one of
@@ -666,66 +943,139 @@ class Profile(props.SyncableHasProperties, actions.ActionProvider):
         else:                return tempMode
 
 
-    def __getHandler(self, ev, evType, mode=None):
-        """Returns a reference to a method of this ``Profile`` instance
-        (defined on the sub-class) which can handle the given
-        :class:`wx.MouseEvent` or :class:`wx.KeyEvent` (the ``ev`` argument).
+    def __getHandler(self,
+                     ev,
+                     evType,
+                     mode=None,
+                     origEvType=None,
+                     direct=False):
+        """Returns a function which will handle the given
+        :class:`wx.MouseEvent` or :class:`wx.KeyEvent` (the ``ev`` argument),
+        or ``None`` if no handlers are found.
 
-        The ``mode`` and ``evType`` arguments may be used to force the lookup
-        of a handler for the specified mode (see the :attr:`mode` property)
-        or event type.
+        If an alternate handler for the mode/event has been specified, it is
+        returned.
 
-        If a handler is not found, the :attr:`__altHandlerMap` map is checked
-        to see if an alternate handler for the mode/event type has been
-        specified.
+        :arg ev:         The event object
+
+        :arg evType:     The event type (e.g. ``'LeftMouseDown'``)
+
+        :arg mode:       Override the default mode with this one. If not
+                         provided, the handler for the current mode (or
+                         temporary mode, if one is active) will be used.
+
+        :arg origEvType: If the ``evType`` is not the actual event that
+                         occurred (e.g. this method has been called to look
+                         up an alternate or fallback handler), the original
+                         event type must be passed in here.
+
+        :arg direct:     If ``False`` (the default), the returned function will
+                         call the standard event handler (a method of the
+                         ``Profile`` sub-class), its fallback handler if it
+                         returns ``False`` and a fallback has been specfiied,
+                         any extra handlers that have been registered for the
+                         event type, and will also call the pre- and post-
+                         event methods. Otherwise, the returned function will
+                         just be the sub-class handler method for the
+                         specified for ``evType/ ``mode``.
         """
 
+        if origEvType is None:
+            origEvType = evType
+
         if mode is None:
             mode = self.__getMode(ev)
 
+        # Lookup any alternate/fallback
+        # handlers for the event
+        alt      = self.__altHandlerMap     .get((mode, evType), None)
+        fallback = self.__fallbackHandlerMap.get((mode, evType), None)
+
         # Is an alternate handler active?
         # Alternate handlers take precedence
         # over default handlers.
-        alt = self.__altHandlerMap.get((mode, evType), None)
-
-        # An alternate handler has
-        # been specified for this
-        # event - look it up.
         if alt is not None:
             altMode, altEvType = alt
-            return self.__getHandler(ev, altEvType, altMode)
+            return self.__getHandler(ev,
+                                     altEvType,
+                                     mode=altMode,
+                                     origEvType=evType)
 
-        # Otherwise search for a default
-        # method which can handle the
-        # specified mode/evtype.
+        # A fallback handler has
+        # been specified for this
+        # event - get a direct ref
+        # to the fallback function
+        if fallback is not None:
+            fbMode, fbEvType = fallback
+            fallback = self.__getHandler(ev,
+                                         fbEvType,
+                                         mode=fbMode,
+                                         origEvType=evType,
+                                         direct=True)
+
+        # Search for a default method
+        # which can handle the specified
+        # mode/evtype.
         if mode is not None:
             handlerName = '_{}Mode{}'.format(mode, evType)
         else:
-            handlerName = '_{}{}'.format(evType[0].lower(),
-                                         evType[1:])
+            handlerName = '_{}{}'.format(evType[0].lower(), evType[1:])
 
-        handler = getattr(self, handlerName, None)
+        defHandler = getattr(self, handlerName, None)
+
+        # If direct=True, we just
+        # return the handler method,
+        # even it there isn't one
+        # defined.
+        if direct:
+            return defHandler
+
+        # Otherwise we return a wrapper
+        # which calls the pre- and post-
+        # methods, and any extra handlers
+        # that have been registered,
+        handlers = []
+
+        # Insert a placeholder for the
+        # default handler, because we
+        # need to check its return value.
+        if defHandler is not None:
+            handlers.append('defHandler')
+
+        handlers.extend(self.__extraHandlers[origEvType].values())
 
         def handlerWrapper(*args, **kwargs):
 
-            # We want current handlers to be
-            # able to access the last event
-            # which occurred. So we call the
-            # current handler, then update
-            # the lastHandler attribute.
+            retval = None
+
             if self.__preEventHandler is not None:
                 self.__preEventHandler(mode, evType)
 
-            handler(*args, **kwargs)
+            for handler in handlers:
+                # Get the return value of the
+                # default handler, and call its
+                # fallback if necessary.
+                if handler == 'defHandler':
+                    retval = defHandler(*args, **kwargs)
+
+                    if retval is False and fallback is not None:
+                        retval = fallback(*args, **kwargs)
+
+                else:
+                    handler(*args, **kwargs)
 
             if self.__postEventHandler is not None:
                 self.__postEventHandler(mode, evType)
 
+            # Store the last event
+            # that was processed
             self.__lastHandler = (mode, evType)
 
-        if handler is not None:
-            log.debug('Handler found for mode {}, event {}'.format(
-                mode, evType))
+            return retval
+
+        if len(handlers) > 0:
+            log.debug('{} Handler(s) found for mode {}, event {}'.format(
+                len(handlers), mode, evType))
             return handlerWrapper
 
         return None
@@ -752,8 +1102,8 @@ class Profile(props.SyncableHasProperties, actions.ActionProvider):
         # produced positive values will now produce
         # negative values.
         if ev.ShiftDown() and \
-           fslplatform.wxPlatform in (fslplatform.WX_MAC_COCOA,
-                                      fslplatform.WX_MAC_CARBON):
+           fwidgets.wxPlatform() in (fwidgets.WX_MAC_COCOA,
+                                     fwidgets.WX_MAC_CARBON):
             wheel = -wheel
 
         log.debug('Mouse wheel event ({}) on {}'.format(
@@ -973,7 +1323,7 @@ class Profile(props.SyncableHasProperties, actions.ActionProvider):
             ev.Skip()
 
 
-class CanvasPanelEventManager(object):
+class CanvasPanelEventManager:
     """This class manages ``wx`` mouse/keyboard events originating from
     :class:`.SliceCanvas` instances contained within :class:`.CanvasPanel`
     views.
@@ -1045,7 +1395,7 @@ class CanvasPanelEventManager(object):
         self.__profile.handleEvent(ev)
 
 
-class PlotPanelEventManager(object):
+class PlotPanelEventManager:
     """This class manages events originating from ``matplotlib`` ``Canvas``
     objects contained within :class:`.PlotPanel` views.
 
