@@ -27,11 +27,12 @@ import fsleyes_widgets.utils.layout            as fsllayout
 import fsleyes.strings                         as strings
 import fsleyes.gl                              as fslgl
 import fsleyes.actions                         as actions
+import fsleyes.editor                          as editor
+import fsleyes.profiles.orthoviewprofile       as orthoviewprofile
+import fsleyes.profiles.orthoeditprofile       as orthoeditprofile
 import fsleyes.gl.ortholabels                  as ortholabels
 import fsleyes.gl.wxglslicecanvas              as slicecanvas
-import fsleyes.controls.cropimagepanel         as cropimagepanel
-import fsleyes.controls.edittransformpanel     as edittransformpanel
-import fsleyes.controls.orthotoolbar           as orthotoolbar
+import fsleyes.controls.locationpanel          as locationpanel
 import fsleyes.controls.orthoedittoolbar       as orthoedittoolbar
 import fsleyes.controls.orthoeditactiontoolbar as orthoeditactiontoolbar
 import fsleyes.controls.orthoeditsettingspanel as orthoeditsettingspanel
@@ -86,19 +87,26 @@ class OrthoPanel(canvaspanel.CanvasPanel):
     **Interaction**
 
 
-    The following interaction profiles are defined for use with the
-    ``OrthoPanel`` (see the :class:`.ViewPanel` for an overview of
-    *profiles*):
+    The :class:`.OrthoPanel` uses the :class:`.OrthoViewProfile` to handle
+    interaction with the user via the mouse and keyboard. Some control panels
+    will activate other interaction profiles while they are open, such as:
 
-    ======== =========================================================
-    ``view`` Viewing/navigation, using the :class:`.OrthoViewProfile`.
+    ========================== ==============================================
+    :class:`.OrthoEditToolBar` Simple editing of :class:`.Image`
+                               overlays, using the
+                               :class:`.OrthoEditProfile` (see also
+                               the :mod:`~fsleyes.editor` package).
 
-    ``edit`` Simple editing of :class:`.Image` overlays, using the
-             :class:`.OrthoEditProfile` (see also the
-             :mod:`~fsleyes.editor` package).
+    :class:`.CropImagePanel`   Allows the user to crop an ``Image`` overlay,
+                               using the :class:`.OrthoCropProfile`.
 
-    ``crop`` Allows the user to crop an ``Image`` overlay.
-    ======== =========================================================
+    :class:`.AnnotationPanel`  Allows the user to draw text and shapes on the
+                               :class:`.OrthoPanel` canvases, using the
+                               :class:`.OrthoAnnotatePanel`.
+    ========================== ==============================================
+
+    See the :class:`.ViewPanel`, and the :mod:`.profiles` package for more
+    information on interaction profiles.
 
 
     **Actions and control panels**
@@ -111,9 +119,6 @@ class OrthoPanel(canvaspanel.CanvasPanel):
        :nosignatures:
 
        toggleEditMode
-       toggleCropMode
-       toggleEditTransformPanel
-       toggleEditPanel
        toggleOrthoToolBar
        resetDisplay
        centreCursor
@@ -126,6 +131,54 @@ class OrthoPanel(canvaspanel.CanvasPanel):
     """
 
 
+    def controlOptions(self, cpType):
+        """Returns some options to be used by :meth:`.ViewPanel.togglePanel`
+        for certain control panel types.
+        """
+        if cpType is locationpanel.LocationPanel:
+            return {'showHistory' : True}
+
+
+    @staticmethod
+    def defaultLayout():
+        """Returns a list of control panel types to be added for the default
+        ortho panel layout.
+        """
+        return ['OverlayDisplayToolBar',
+                'OrthoToolBar',
+                'OverlayListPanel',
+                'LocationPanel']
+
+
+    @staticmethod
+    def controlOrder():
+        """Returns a list of control panel names, specifying the order in
+        which they should appear in the  FSLeyes ortho panel settings menu.
+        """
+        return ['OverlayListPanel',
+                'LocationPanel',
+                'OverlayInfoPanel',
+                'OverlayDisplayPanel',
+                'CanvasSettingsPanel',
+                'AtlasPanel',
+                'OverlayDisplayToolBar',
+                'OrthoToolBar',
+                'FileTreePanel']
+
+
+    @staticmethod
+    def toolOrder():
+        """Returns a list of tool names, specifying the order in which they
+        should appear in the FSLeyes ortho panel settings menu.
+        """
+        return ['CropImageAction',
+                'EditTransformAction',
+                'SampleLineAction',
+                'LoadAnnotationsAction',
+                'SaveAnnotationsAction',
+                'PearsonCorrelateAction']
+
+
     def __init__(self, parent, overlayList, displayCtx, frame):
         """Create an ``OrthoPanel``.
 
@@ -147,8 +200,10 @@ class OrthoPanel(canvaspanel.CanvasPanel):
         name         = self.name
         contentPanel = self.contentPanel
 
-        # The canvases themselves - each one displays a
-        # slice along each of the three world axes
+        # The canvases themselves - each one displays
+        # a slice along each of the three world axes.
+        # They are laid out in __refreshLayout
+        self.__canvasSizer = None
         self.__xcanvas = slicecanvas.WXGLSliceCanvas(contentPanel,
                                                      overlayList,
                                                      displayCtx,
@@ -170,11 +225,24 @@ class OrthoPanel(canvaspanel.CanvasPanel):
             self.__ycanvas,
             self.__zcanvas)
 
-        # If an edit menu is added when in
-        # 'edit' profile (see __profileChanged),
-        # its name is stored here.
+        # Edit mode is entered via toggleEditMode,
+        # which displays an edit toolbar. But we
+        # do a bit more, in adding the edit action
+        # toolbar, and edit menu. In order to keep
+        # the menu state in sync with reality, we
+        # need to listen for changes to the
+        # interaction proflle, and store a ref
+        # to the edit menu so we can delete it later.
+        self.events.register(self.name, self.__profileChanged, 'profile')
         self.__editMenuTitle = None
 
+        # The OrthoOpts and DisplayContext objects
+        # contains the properties that the user
+        # interacts with, but the slice canvases
+        # are controlled by properties on a
+        # SliceCanvsOpts object (one for each
+        # canvas). So we bind properties of these
+        # objects together.
         xopts = self.__xcanvas.opts
         yopts = self.__ycanvas.opts
         zopts = self.__zcanvas.opts
@@ -211,6 +279,13 @@ class OrthoPanel(canvaspanel.CanvasPanel):
         yopts.bindProps('highDpi',      sceneOpts)
         zopts.bindProps('highDpi',      sceneOpts)
 
+        xopts.bindProps('invertX', sceneOpts, 'invertXHorizontal')
+        xopts.bindProps('invertY', sceneOpts, 'invertXVertical')
+        yopts.bindProps('invertX', sceneOpts, 'invertYHorizontal')
+        yopts.bindProps('invertY', sceneOpts, 'invertYVertical')
+        zopts.bindProps('invertX', sceneOpts, 'invertZHorizontal')
+        zopts.bindProps('invertY', sceneOpts, 'invertZVertical')
+
         # Callbacks for ortho panel layout options
         sceneOpts.addListener('layout', name, self.__refreshLayout)
 
@@ -250,16 +325,15 @@ class OrthoPanel(canvaspanel.CanvasPanel):
         sceneOpts.addListener('fgColour',    name, refresh, weak=False)
         sceneOpts.addListener('showLabels',  name, refresh, weak=False)
 
-        self.addListener('profile', name, self.__profileChanged)
-
-        from fsleyes.actions.correlate import PearsonCorrelateAction
-
-        self.__pCorrAction = PearsonCorrelateAction(
-            self.overlayList,
-            self.displayCtx,
-            self)
-
-        self.pearsonCorrelation.bindProps('enabled', self.__pCorrAction)
+        # The lastFocusedCanvas method allows
+        # a ref to the most recently focused
+        # canvas to be obtained. We listen
+        # for focus events on each canvas,
+        # and update this ref each time
+        self.__focusedCanvas = None
+        self.__xcanvas.Bind(wx.EVT_SET_FOCUS, self.__onCanvasFocus)
+        self.__ycanvas.Bind(wx.EVT_SET_FOCUS, self.__onCanvasFocus)
+        self.__zcanvas.Bind(wx.EVT_SET_FOCUS, self.__onCanvasFocus)
 
         # Call the __onResize method to refresh
         # the slice canvases when the canvas
@@ -272,7 +346,7 @@ class OrthoPanel(canvaspanel.CanvasPanel):
         self.__refreshLayout(refresh=False)
         self.__overlayListChanged()
         self.centrePanelLayout()
-        self.initProfile()
+        self.initProfile(orthoviewprofile.OrthoViewProfile)
 
 
     def destroy(self):
@@ -304,92 +378,64 @@ class OrthoPanel(canvaspanel.CanvasPanel):
         self.__ycanvas.destroy()
         self.__zcanvas.destroy()
         self.__removeEditMenu()
-        self.__pCorrAction.destroy()
 
         contentPanel.Unbind(wx.EVT_SIZE)
 
-        self.__xcanvas     = None
-        self.__ycanvas     = None
-        self.__zcanvas     = None
-        self.__labelMgr    = None
-        self.__pCorrAction = None
+        self.__xcanvas       = None
+        self.__ycanvas       = None
+        self.__zcanvas       = None
+        self.__focusedCanvas = None
+        self.__labelMgr      = None
 
         canvaspanel.CanvasPanel.destroy(self)
 
 
-    @actions.toggleControlAction(orthotoolbar.OrthoToolBar)
-    def toggleOrthoToolBar(self):
-        """Shows/hides an :class:`.OrthoToolBar`. See
-        :meth:`.ViewPanel.togglePanel`.
-        """
-        self.togglePanel(orthotoolbar.OrthoToolBar, ortho=self)
-
-
     @actions.toggleControlAction(orthoedittoolbar.OrthoEditToolBar)
     def toggleEditMode(self):
-        """Toggles the :attr:`.ViewPanel.profile` between ``'view'`` and
-        ``'edit'``. See :meth:`__profileChanged`.
+        """Shows/hides an :class:`.OrthoEditToolBar`. This causes the
+        interaction profile to be changed, and a call to
+        :meth:`__profileChanged`, which makes a few other changes to the
+        interface.
+        """
+        self.togglePanel(orthoedittoolbar.OrthoEditToolBar)
+        self.togglePanel(orthoeditactiontoolbar.OrthoEditActionToolBar,
+                         location=wx.LEFT)
+
+        if self.toggleEditMode.toggled and \
+           self.isPanelOpen(orthoeditsettingspanel.OrthoEditSettingsPanel):
+            self.togglePanel(orthoeditsettingspanel.OrthoEditSettingsPanel)
+
+
+    def __profileChanged(self, inst, topic, value):
+        """Called when the interaction profile changes (see
+        :meth:`.ViewPanel.events`). If entering or exiting edit mode, an
+        edit menu is added/removed from the menu bar.
         """
 
-        if self.profile == 'view': self.profile = 'edit'
-        else:                      self.profile = 'view'
+        old, new = value
 
-
-    @actions.toggleControlAction(cropimagepanel.CropImagePanel)
-    def toggleCropMode(self):
-        """Toggles the :attr:`.ViewPanel.profile` between ``'view'`` and
-        ``'crop'``. See :meth:`__profileChanged`.
-        """
-
-        if self.profile == 'view': self.profile = 'crop'
-        else:                      self.profile = 'view'
-
-
-    @actions.toggleControlAction(edittransformpanel.EditTransformPanel)
-    def toggleEditTransformPanel(self):
-        """Shows/hides an :class:`.EditTransformPanel`. See
-        :meth:`.ViewPanel.togglePanel`.
-        """
-
-        self.togglePanel(edittransformpanel.EditTransformPanel,
-                         floatPane=True,
-                         floatOnly=True,
-                         closeable=False,
-                         ortho=self)
-
-
-    @actions.toggleControlAction(orthoeditsettingspanel.OrthoEditSettingsPanel)
-    def toggleEditPanel(self, floatPane=False):
-        """Shows/hides an :class:`.OrthoEditSettingsPanel`. See
-        :meth:`.ViewPanel.togglePanel`.
-        """
-        self.togglePanel(orthoeditsettingspanel.OrthoEditSettingsPanel,
-                         ortho=self,
-                         floatPane=floatPane)
-
-
-    @actions.action
-    def pearsonCorrelation(self):
-        """Executes a :class:`.PearsonCorrelateAction`. """
-        self.__pCorrAction()
+        if new is orthoeditprofile.OrthoEditProfile:
+            self.__addEditMenu()
+        elif old is orthoeditprofile.OrthoEditProfile:
+            self.__removeEditMenu()
 
 
     @actions.action
     def resetDisplay(self):
         """Calls :meth:`.OrthoViewProfile.resetDisplay`. """
-        self.getCurrentProfile().resetDisplay()
+        self.currentProfile.resetDisplay()
 
 
     @actions.action
     def centreCursor(self):
         """Calls :meth:`.OrthoViewProfile.centreCursor`. """
-        self.getCurrentProfile().centreCursor()
+        self.currentProfile.centreCursor()
 
 
     @actions.action
     def centreCursorWorld(self):
         """Calls :meth:`.OrthoViewProfile.centreCursorWorld`. """
-        self.getCurrentProfile().centreCursorWorld()
+        self.currentProfile.centreCursorWorld()
 
 
     @actions.toggleAction
@@ -452,55 +498,25 @@ class OrthoPanel(canvaspanel.CanvasPanel):
                    self.toggleCursor,
                    self.toggleXCanvas,
                    self.toggleYCanvas,
-                   self.toggleZCanvas,
-                   None,
-                   self.toggleOverlayList,
-                   self.toggleLocationPanel,
-                   self.toggleOverlayInfo,
-                   self.toggleDisplayPanel,
-                   self.toggleCanvasSettingsPanel,
-                   self.toggleAtlasPanel,
-                   self.toggleDisplayToolBar,
-                   self.toggleOrthoToolBar,
-                   self.toggleLookupTablePanel,
-                   self.toggleClusterPanel,
-                   self.toggleClassificationPanel,
-                   self.removeAllPanels]
+                   self.toggleZCanvas]
 
-        def makeTuples(actionz):
-
-            tuples = []
-
-            for a in actionz:
-                if isinstance(a, actions.Action):
-                    tuples.append((a.__name__, a))
-
-                elif isinstance(a, tuple):
-                    tuples.append((a[0], makeTuples(a[1])))
-
-                elif a is None:
-                    tuples.append((None, None))
-
-            return tuples
-
-        return makeTuples(actionz)
+        names = [a.actionName if a is not None else None for a in actionz]
+        return list(zip(names, actionz))
 
 
     def getTools(self):
-        """Returns a list of methods to be added to the ``FSLeyesFrame`` Tools menu
-        for ``OrthoPanel`` views.
+        """Returns a list of methods to be added to the ``FSLeyesFrame`` Tools
+        menu for ``OrthoPanel`` views.
         """
-        return [self.toggleEditMode,
-                self.toggleCropMode,
-                self.toggleEditTransformPanel,
-                self.pearsonCorrelation]
+        return [self.toggleEditMode]
 
 
     def getGLCanvases(self):
         """Returns all of the :class:`.SliceCanvas` instances contained
         within this ``OrthoPanel``.
         """
-        return [self.__xcanvas, self.__ycanvas, self.__zcanvas]
+        canvas = [self.__xcanvas, self.__ycanvas, self.__zcanvas]
+        return [c for c in canvas if c is not None]
 
 
     def getXCanvas(self):
@@ -521,58 +537,18 @@ class OrthoPanel(canvaspanel.CanvasPanel):
         return self.__zcanvas
 
 
-    def __profileChanged(self, *a):
-        """Called when the :attr:`.ViewPanel.profile` changes. If ``'edit'``
-        mode has been enabled, :class:`.OrthEditToolBar` and
-        :class:`.OrthEditActionToolBar` toolbars are added as control panels,
-        and an "edit" menu is added to the :class:`.FSLeyesFrame` (if there
-        is one).
+    def lastFocusedCanvas(self):
+        """Return a reference to the :class:`.SliceCanvas` which most recently
+        had focus. Will be ``None`` if called before any canvas gains focus.
         """
+        return self.__focusedCanvas
 
-        CropImagePanel         = cropimagepanel.CropImagePanel
-        OrthoEditToolBar       = orthoedittoolbar.OrthoEditToolBar
-        OrthoEditActionToolBar = orthoeditactiontoolbar.OrthoEditActionToolBar
-        OrthoEditSettingsPanel = orthoeditsettingspanel.OrthoEditSettingsPanel
 
-        cropPanelOpen          = self.isPanelOpen(CropImagePanel)
-        editToolBarOpen        = self.isPanelOpen(OrthoEditToolBar)
-        editActionToolBarOpen  = self.isPanelOpen(OrthoEditActionToolBar)
-        editPanelOpen          = self.isPanelOpen(OrthoEditSettingsPanel)
-
-        inEdit                 = self.profile == 'edit'
-        inCrop                 = self.profile == 'crop'
-
-        # Toggle toolbars if they are open but should
-        # be closed, or closed but should be open
-        if (not editToolBarOpen) and      inEdit or \
-                editToolBarOpen  and (not inEdit):
-            self.togglePanel(orthoedittoolbar.OrthoEditToolBar, ortho=self)
-
-        if (not editActionToolBarOpen) and      inEdit or \
-                editActionToolBarOpen  and (not inEdit):
-            self.togglePanel(orthoeditactiontoolbar.OrthoEditActionToolBar,
-                             ortho=self,
-                             location=wx.LEFT)
-
-        if (not cropPanelOpen) and      inCrop or \
-                cropPanelOpen  and (not inCrop):
-            self.togglePanel(cropimagepanel.CropImagePanel,
-                             ortho=self,
-                             floatPane=True,
-                             floatOnly=True,
-                             closeable=False,
-                             floatPos=(0.85, 0.3))
-
-        # Don't open edit panel by default,
-        # but close it when we leave edit mode
-        if editPanelOpen and (not inEdit):
-            self.togglePanel(orthoeditsettingspanel.OrthoEditSettingsPanel)
-
-        # It's unlikely, but an OrthoPanel might be
-        # created without a ref to a FSLeyesFrame.
-        if self.frame is not None:
-            if inEdit: self.__addEditMenu()
-            else:      self.__removeEditMenu()
+    def __onCanvasFocus(self, ev):
+        """Called when any :class:`.SliceCanvas` gains focus. Updates the
+        last focused canvas reference.
+        """
+        self.__focusedCanvas = ev.GetEventObject()
 
 
     def __addEditMenu(self):
@@ -585,7 +561,7 @@ class OrthoPanel(canvaspanel.CanvasPanel):
         menuName = strings.labels[self, 'editMenu']
         menuName = menuName.format(frame.getViewPanelID(self))
         menuBar  = frame.GetMenuBar()
-        profile  = self.getCurrentProfile()
+        profile  = self.currentProfile
         idx      = menuBar.FindMenu(menuName)
 
         if  idx == wx.NOT_FOUND: editMenu = None
@@ -606,9 +582,9 @@ class OrthoPanel(canvaspanel.CanvasPanel):
                    'clearSelection',
                    'fillSelection',
                    'eraseSelection',
-                   'copySelection',
-                   'pasteSelection',
-                   'invertSelection']
+                   'invertSelection',
+                   'copyPasteData',
+                   'copyPasteSelection']
 
         frame.populateMenu(editMenu,
                            profile,
@@ -625,7 +601,6 @@ class OrthoPanel(canvaspanel.CanvasPanel):
                            runOnIdle=True)
 
 
-
     def __removeEditMenu(self):
         """Called by :meth:`__profleChanged` when the
         :attr:`.ViewPanel.profile` is changed from ``'edit'``. If an edit
@@ -709,18 +684,26 @@ class OrthoPanel(canvaspanel.CanvasPanel):
 
         if selOverlay is not None:
 
-            display = self.displayCtx.getDisplay(selOverlay)
-            isImage = isinstance(selOverlay, fslimage.Image) and \
-                      display.overlayType in ('volume', 'mask', 'label')
+            display    = self.displayCtx.getDisplay(selOverlay)
+            isImage    = isinstance(selOverlay, fslimage.Image) and \
+                             display.overlayType in ('volume',
+                                                     'mask',
+                                                     'label',
+                                                     'mip')
+            isEditable = editor.isEditable(selOverlay, self.displayCtx)
         else:
-            isImage = False
+            isImage    = False
+            isEditable = False
+
+        # Kill edit mode if a non-
+        # image has been selected
+        if (not isEditable) and self.toggleEditMode.toggled:
+            self.toggleEditMode()
 
         self.resetDisplay            .enabled = haveOverlays
         self.centreCursor            .enabled = haveOverlays
         self.centreCursorWorld       .enabled = haveOverlays
-        self.toggleEditMode          .enabled = isImage
-        self.toggleEditTransformPanel.enabled = isImage
-        self.toggleCropMode          .enabled = isImage
+        self.toggleEditMode          .enabled = isEditable
 
 
     def __onResize(self, ev):
@@ -782,8 +765,8 @@ class OrthoPanel(canvaspanel.CanvasPanel):
                                     height)
 
         for canvas, size in zip(canvases, sizes):
-            canvas.SetMinSize(size)
-            canvas.SetMaxSize(size)
+            canvas.SetMinSize((round(size[0]), round(size[1])))
+            canvas.SetMaxSize((round(size[0]), round(size[1])))
 
 
     def __refreshLayout(self, *a, **kwa):
@@ -833,6 +816,10 @@ class OrthoPanel(canvaspanel.CanvasPanel):
         elif layout == 'vertical':   nrows, ncols = 3, 1
         elif layout == 'grid':       nrows, ncols = 2, 2
 
+        if self.__canvasSizer is not None:
+            self.__canvasSizer.Clear()
+            self.__canvasSizer = None
+
         self.__canvasSizer = wx.FlexGridSizer(nrows, ncols, 0, 0)
 
         # The rows/columns that contain
@@ -845,10 +832,8 @@ class OrthoPanel(canvaspanel.CanvasPanel):
             canvases.append((0, 0))
 
         # Add all those widgets to the grid sizer
-        flag = wx.ALIGN_CENTRE_HORIZONTAL | wx.ALIGN_CENTRE_VERTICAL
-
         for c in canvases:
-            self.__canvasSizer.Add(c, flag=flag | wx.EXPAND)
+            self.__canvasSizer.Add(c, flag=wx.EXPAND)
 
         self.contentPanel.SetSizer(self.__canvasSizer)
 
